/******************************************************************************
*                                                                     
*       Copyright (C) 2011 Freescale Semiconductor, Inc.              
*       All Rights Reserved								              
*														              
* Filename:     adc.h                
*														              
* Revision:      										              
*														              
* Functions:    
*         
* Description:  ADC16 Header file, function prototypes, structures and defines
*
* Ported to Mini @b17539
* 
* Notes:        
*
******************************************************************************/

#ifndef _ADC_H
#define _ADC_H

#include "common.h"

/* Typedefs */
typedef struct _CALBLOCK
{
	unsigned long int clpd;
	unsigned long int clps;
	unsigned long int clp4;
	unsigned long int clp3;
	unsigned long int clp2;
	unsigned long int clp1;
	unsigned long int clp0;
	unsigned long int clmd;
	unsigned long int clms;
	unsigned long int clm4;
	unsigned long int clm3;
	unsigned long int clm2;
	unsigned long int clm1;
	unsigned long int clm0;
} CALBLOCK;

typedef struct _ADC_MODE
{
	unsigned char complete_flag;	//conversion complete flag
	unsigned char ien;	//interrupt enable
	unsigned char channel;	//channel selection
	unsigned char lpw;	//normal power or low power, 0: normal power, 1: low power
	unsigned char mode;	//conversion mode, 0: 8bit, 1: 12bit, 2: 10bit
	unsigned char clkdiv;	//0: div by 1, 1: div by 2, 2: div by 4, 3: div by 8
	unsigned char clksel;	//0: bus clock, 1: bus clock/2, 2: ALTCLK, 3: ADACK
	unsigned char adacken;	//if enable ADACK, 0: ADACK disable, 1: ADACK enable
	unsigned char high_speed;	//normal conversion or high speed conversion, 0: normal conversion, 1: high speed conversion
	unsigned char longsample;	//short or long sample time, 0: short sample, 1: long sample
	unsigned char sample_time;	//long sample time select, 0: 20 extra ADCK cycles, 1: 12 extra ADCK cycles, 2: 6 extra ADCK cycles, 3: 2 extra ADCK cycles
	unsigned char adc_muxsel;	//0: ADxxA channels selected, 1: ADxxB channels selected
	unsigned char trigger_src;	//software or hardware trigger, 0: software trigger, 1: PDB trigger, 2: LPTMR trigger
	unsigned char cmpen;	//compare function enable, 0: compare disable, 1: compare enable
	unsigned char cmpgt;	//0: compare less than, 1: compare great than
	unsigned char cmprg;	//0: compare range disable, 1: compare range enable
	unsigned char dmaen;	//0: DMA disable, 1: DMA enable
	unsigned char refsel;	//voltage reference select, 0: VREFH/VREFL, 1: VALTH/VALTL
	unsigned char conten;	//single or continuous conversion select, 0: single conversion, 1: continuous conversion
	unsigned char averen;	//hardware average enable, 0: disable, 1: enable
	unsigned char aversel;	//hardware average select, 0: 4 sample, 1: 8 sample, 2: 16 sample, 3: 32 sample
	unsigned int resulta;	//conversion result a
	unsigned int resultb;	//conversion result b
	unsigned int cmp_val_one;	//compare value one
	unsigned int cmp_val_two;	//compare value two
	unsigned int offset;	//offset correction register
	CALBLOCK calblock;	//calibration block
} ADC_MODE;

/* ADC input channels */ 
typedef enum {
	ADC_CH_SE2 = 2,
	ADC_CH_SE3,
	ADC_CH_12bitDAC,
	ADC_CH_SE8 = 8,
	ADC_CH_SE9,
	ADC_CH_SE10,
	ADC_CH_SE11,
	ADC_CH_SE12,
	ADC_CH_SE13,
	ADC_CH_SE14,
	ADC_CH_SE15,
	ADC_CH_SE16,
	ADC_CH_SE17,
	ADC_CH_SE18,
	ADC_CH_SE19,
	ADC_CH_SE20,
	ADC_CH_SE21,
	ADC_CH_SE22,
	ADC_CH_6bitDAC,
	ADC_CH_TempSensor = 26,
	ADC_CH_VREFO,
	ADC_CH_VREFH = 29,
	ADC_CH_VREFL,
	ADC_CH_DISABLE
} ADC_CHANNEL;

/* ADC reference */
typedef enum {
	ADC_REF_VREFHL = 0,
	ADC_REF_VREFO
} ADC_REF;

/* 
 * ADC hardware trigger select
 * configure ADCx_SC2[ADTRG] bit and SIM_SOPT7[ADTRGS] bit
 */
typedef enum {
	ADC_SOFT_TRIG = 0,
	ADC_TRIG_PDB,
	ADC_TRIG_LPTMR0
} ADC_TRIG_SRC;

/* ADC power mode */
typedef enum {
	ADC_NORMAL_POWER_MODE = 0,
	ADC_LOW_POWER_MODE
} ADC_POWER_MODE;

/* ADC clock divide ratio */
typedef enum {
	ADC_CLOCK_DIV1 = 0,
	ADC_CLOCK_DIV2,
	ADC_CLOCK_DIV4,
	ADC_CLOCK_DIV8
} ADC_CLOCK_DIV;

/* sample time configuration */
typedef enum {
	ADC_SHORT_SAMPLE_TIME = 0,
	ADC_LONG_SAMPLE_TIME
} ADC_SAMPLE_TIME;

/* conversion mode */
typedef enum {
	ADC_8BIT_CONVERSION = 0,
	ADC_12BIT_CONVERSION,
	ADC_10BIT_CONVERSION
} ADC_CONVERSION_MODE;

/* ADC input clock select */
typedef enum {
	ADC_BUS_CLOCK = 0,
	ADC_BUS_CLOCK_DIV2,
	ADC_ALTCLK,
	ADC_ADACK	
} ADC_INCLK_SEL;

/* ADC Mux select */
typedef enum {
	ADC_MUX_ADXXA = 0,
	ADC_MUX_ADXXB
} ADC_MUX_SEL;

/* ADC conversion speed configuration */
typedef enum {
	ADC_NORMAL_SPEED = 0,
	ADC_HIGH_SPEED	
} ADC_SPEED_MODE;

/* ADC sample time select */
typedef enum {
	ADC_20EXTRA_ADCK = 0,
	ADC_12EXTRA_ADCK,
	ADC_6EXTRA_ADCK,
	ADC_2EXTRA_ADCK
} ADC_SAMPLE_TIME_SEL;

/* hardware average select */
typedef enum {
	ADC_HARD_AVEG_4SAMPLE = 0,
	ADC_HARD_AVEG_8SAMPLE,
	ADC_HARD_AVEG_16SAMPLE,
	ADC_HARD_AVEG_32SAMPLE
} ADC_HARD_AVEG_SEL;

/* compare great to less than */
typedef enum {
	ADC_COMPARE_LESS = 0, 
	ADC_COMPARE_GT
} ADC_COMPARE_GT_LESS;

/* single or continous conversion */
typedef enum {
	ADC_SINGLE_CONVERSION = 0,
	ADC_CONTINUOUS_CONVERSION
} ADC_SINLE_OR_CONT;

/* Macros */
#define DEBUG

#define ADC_ADACK_DISABLE 0
#define ADC_ADACK_ENABLE 1

#define ADC_COMPARE_DISABLE 0
#define ADC_COMPARE_ENABLE 1

#define ADC_COMPARE_RANGE_DISABLE 0
#define ADC_COMPARE_RANGE_ENABLE 1

#define ADC_DMA_DISABLE 0
#define ADC_DMA_ENABLE 1

#define ADC_HARD_AVEG_DISABLE 0
#define ADC_HARD_AVEG_ENABLE 1

/* Function declarations */
/* ADC module initialization */
unsigned char adc_init(ADC_MODE *modeptr);

/* calibrate ADC module */
unsigned char calibrate_adc(ADC_MODE *modeptr);

/* Start ADC module */
unsigned char adc_start(ADC_MODE *modeptr, unsigned char ch);

/* Stop ADC module */
void adc_stop(ADC_MODE *modeptr);

/* get adc result */
unsigned char get_adc_result(ADC_MODE *modeptr);

/* store calibration block */
void store_calblk(ADC_MODE *modeptr);

/* dump calibration block */
void dump_calblk(ADC_MODE *modeptr);

#endif /* _ADC_H */

