#ifndef _DEBUG_H_
#define _DEBUG_H_

// LED defines
#define LED_ON_RED       PTB_D = PTB_D & (~0x40)
#define LED_OFF_RED      PTB_D = PTB_D | 0x40

#define LED_ON_GREEN     PTB_D = PTB_D & (~0x20) 
#define LED_OFF_GREEN    PTB_D = PTB_D | 0x20

// Define for Clock Selection on CLOCKOUT
typedef enum {
 CLKOUT_OFF     =0x0,
 OSCOUT1_CLK 	=0x1,
 OSCOUT2_CLK 	=0x2,
 MCGOUT  	    =0x3,
 CPUCLK 	    =0x4,
 BUSCLK    	    =0x5,
 LPOCLK         =0x6,
 LPTMR0OUT      =0x7
} SIM_CLKOUT_CS_T;

typedef enum {
 DIV_BY_1       =0x0,
 DIV_BY_2 		=0x1,
 DIV_BY_4 		=0x2,
 DIV_BY_8  		=0x3,
 DIV_BY_16		=0x4,
 DIV_BY_32   	=0x5,
 DIV_BY_64      =0x6,
 DIV_BY_128     =0x7
} SIM_CLKOUT_DIV_T;

typedef enum {
 CLKOUT_PTA5     =0x0,
 CLKOUT_PTC3	 =0x1
} SIM_CLKOUT_PAD_T;

/**********STARTFUNC**************************************************
 * Name:  sim_clkout_config
 * Purpose: Bringing a particular clock on CLKOUT for observability
 * Algorithm:
 * Inputs:	clk : Clock which has to be observed
 * 		div : divider for the clkout pin
 * 		pad : selects the pad on which the CLKOUT will come
 * Outputs: NONE
 * Return value: NONE
 * Assumptions : NONE
 * Child Functions : NONE
 *
 ***********ENDFUNC***************************************************/
void sim_clkout_config(SIM_CLKOUT_CS_T clk, SIM_CLKOUT_DIV_T div, SIM_CLKOUT_PAD_T pad);

/**********STARTFUNC**************************************************
 * Name:  LED_RED_SETUP
 * Purpose: Does setup for RED LED
 * Algorithm: 
 * Inputs:	None
 * Outputs: NONE 
 * Return value: NONE
 * Assumptions : NONE
 * Child Functions : NONE
 *
 ***********ENDFUNC***************************************************/
void LED_RED_SETUP(void) ;

/**********STARTFUNC**************************************************
 * Name:  LED_GREEN_SETUP
 * Purpose: Does setup for Green LED
 * Algorithm: 
 * Inputs:	None
 * Outputs: NONE 
 * Return value: NONE
 * Assumptions : NONE
 * Child Functions : NONE
 *
 ***********ENDFUNC***************************************************/
void LED_GREEN_SETUP(void) ;

#endif // _DEBUG_H_

