/******************************************************************************
*                                                                     
*       Copyright (C) 2009 Freescale Semiconductor, Inc.              
*       All Rights Reserved								              
*														              
* Filename:     adc.c                
*														              
* Revision:      										              
*														              
* Functions:    
*         
* Description:  ADC C file, function implementations
*
* Ported to Mini, @b17539
* 
* Notes:        
*
******************************************************************************/

#include "adc.h"

/*
 * ADC pin multiplex 
 * ADC_SE2 - PTA4	ADC_SE3 - PTA5	ADC_SE8 - PTA8	ADC_SE9 - PTD2
 * ADC_SE10 - PTD3	ADC_SE11 - PTD4	ADC_SE12 - PTD5	ADC_SE13 - PTA7
 * ADC_SE14 - PTD6	ADC_SE15 - PTD7	ADC_SE16 - PTE3	ADC_SE17 - PTB2
 * ADC_SE18 - PTB3	ADC_SE19 - PTE4	ADC_SE20 - PTE5	ADC_SE21 - PTE6
 * ADC_SE22 - PTE7
 */

/* ADC module initialization */
unsigned char adc_init(ADC_MODE *modeptr)
{
	ADC_MemMapPtr adcptr = ADC0_BASE_PTR;
	OSC_MemMapPtr oscptr = OSC2_BASE_PTR;
	SIM_MemMapPtr simptr = SIM_BASE_PTR;
	
	/* enable ADC clock gate */
	simptr->SCGC2 |= SIM_SCGC2_ADC_MASK;

	/* interrupt enable */
	if(modeptr->ien)
		adcptr->SC1[0] |= ADC_SC1_AIEN_MASK;
	else
		adcptr->SC1[0] &= ~ADC_SC1_AIEN_MASK;
		
	/* normal or low power mode */
	if(modeptr->lpw)
		adcptr->CFG1 |= ADC_CFG1_ADLPC_MASK; 
	else
		adcptr->CFG1 &= ~ADC_CFG1_ADLPC_MASK;
		
	/* mode selection */
	adcptr->CFG1 |= ADC_CFG1_MODE(modeptr->mode);	

	/* clk divide and input select */
	adcptr->CFG1 |= ADC_CFG1_ADIV(modeptr->clkdiv);
	adcptr->CFG1 |= ADC_CFG1_ADICLK(modeptr->clksel);
			
	if(ADC_ALTCLK == modeptr->clksel)
		oscptr->CR |= OSC_CR_ERCLKEN_MASK;
	
	/* ADACK enable */
	if(modeptr->adacken)
		adcptr->CFG2 |= ADC_CFG2_ADACKEN_MASK;
	else
		adcptr->CFG2 &= ~ADC_CFG2_ADACKEN_MASK;

	/* high speed or normal conversion */
	if(modeptr->high_speed)
		adcptr->CFG2 |= ADC_CFG2_ADHSC_MASK;
	else
		adcptr->CFG2 &= ~ADC_CFG2_ADHSC_MASK;		
	
	/* short or long sample time */
	if(modeptr->longsample)
	{	//long sample
		adcptr->CFG1 |= ADC_CFG1_ADLSMP_MASK;		
		/* long sample time select */
		adcptr->CFG2 |= ADC_CFG2_ADLSTS(modeptr->sample_time);
	}
	else
	{	//short sample
		adcptr->CFG1 &= ~ADC_CFG1_ADLSMP_MASK;
	}
	
	if(modeptr->adc_muxsel)
		adcptr->CFG2 |= ADC_CFG2_MUXSEL_MASK;
	else
		adcptr->CFG2 &= ~ADC_CFG2_MUXSEL_MASK;
	
	/* hardware trigger */
	switch(modeptr->trigger_src)
	{	
	case ADC_SOFT_TRIG:
		adcptr->SC2 &= ~ADC_SC2_ADTRG_MASK;
		break;
	case ADC_TRIG_PDB:
		//configure PDB as trigger src
		adcptr->SC2 |= ADC_SC2_ADTRG_MASK;
		simptr->SOPT7 &= ~SIM_SOPT7_ADTRGS_MASK;
		break;
	case ADC_TRIG_LPTMR0:
		/* configure LPTMR as trigger src */
		adcptr->SC2 |= ADC_SC2_ADTRG_MASK;
		simptr->SOPT7 |= SIM_SOPT7_ADTRGS_MASK;
		break;
	default:
		return -1;
	}	
	
	/* compare function enabled */
	if(modeptr->cmpen)
	{	//compare enable
		adcptr->SC2 |= ADC_SC2_ACFE_MASK;
		if(modeptr->cmpgt)	
			adcptr->SC2 |= ADC_SC2_ACFGT_MASK;
		else
			adcptr->SC2 &= ~ADC_SC2_ACFGT_MASK;
		if(modeptr->cmprg)
		{	//compare range enable
			adcptr->SC2 |= ADC_SC2_ACREN_MASK;
			adcptr->CV1 = modeptr->cmp_val_one;
			adcptr->CV2 = modeptr->cmp_val_two;
		}
		else
		{	//compare range disable
			adcptr->SC2 &= ~ADC_SC2_ACREN_MASK;
			adcptr->CV1 = modeptr->cmp_val_one;
		}
	}
	else
		adcptr->SC2 &= ~ADC_SC2_ACFE_MASK;
		
	if(modeptr->dmaen)
		adcptr->SC2 |= ADC_SC2_DMAEN_MASK;
	else
		adcptr->SC2 &= ~ADC_SC2_DMAEN_MASK;
		
	/* voltage reference select */
	adcptr->SC2 |= ADC_SC2_REFSEL(modeptr->refsel);
	
	/* single or continuous conversion */
	if(modeptr->conten)
		adcptr->SC3 |= ADC_SC3_ADCO_MASK;
	else
		adcptr->SC3 &= ~ADC_SC3_ADCO_MASK;
	
	/* hardware average enabled */
	if(modeptr->averen)
	{
		adcptr->SC3 |= ADC_SC3_AVGE_MASK;
		adcptr->SC3 |= ADC_SC3_AVGS(modeptr->aversel);
	}
	else
		adcptr->SC3 &= ~ADC_SC3_AVGE_MASK;
	
	if(modeptr->offset)
		adcptr->OFS = ADC_OFS_OFS(modeptr->offset);
	else
		adcptr->OFS = 0x0;
	
	return 0;
}

/* calibrate ADC module */
/* For maximum accuracy, set hardware average to maximum (AVGE=1, AVGS=11) , fADCK <= 4MHz, VREFH=VDDA, use software trigger, otherwise CALF gets set */
unsigned char calibrate_adc(ADC_MODE *modeptr)
{
	ADC_MemMapPtr adcptr = ADC0_BASE_PTR;
	unsigned int cal_temp = 0;
	
	adc_init(modeptr);
	
	adcptr->SC3 |= ADC_SC3_CALF_MASK;		// clear any existing calibration failed flag
	adcptr->SC3 |= ADC_SC3_CAL_MASK;			// Start Calibration
	
	/* 
	 * polling CAL flag will be better than polling COCO flag, in case we use enable interrupt and
	 * in ISR we read ADCA to get result, this will clear COCO flag, which ends up we wait endless 
	 * in the loop
	 * */
	while(adcptr->SC3 & ADC_SC3_CAL_MASK)
	{
		
	}
	
#if 0 
	while(0x0 == (adcptr->SC1[0] & ADC_SC1_COCO_MASK)){		// wait for calibration to finish
	};	
	adcptr->SC1[0] |= ADC_SC1_COCO_MASK;	//clear COCO flag
#endif
	
	if(adcptr->SC3 & ADC_SC3_CALF_MASK)
		// There was a fault during calibration
		return 1;
	
	// Calibration was successful
	
	// To complete calibration, the user must generate the gain calibration values 
	// using the following procedure:
	
	// Initialize (clear) a 16b variable in RAM.
	cal_temp = 0;
	
	// Add the following plus-side calibration results CLP0, CLP1, CLP2, CLP3, CLP4, and CLPS to the variable.
	cal_temp += adcptr->CLP0;
	cal_temp += adcptr->CLP1;
	cal_temp += adcptr->CLP2;
	cal_temp += adcptr->CLP3;
	cal_temp += adcptr->CLP4;
	cal_temp += adcptr->CLPS;
	//cal_temp += adcptr->CLPD;

	// Divide the variable by two.
	cal_temp >>= 1;

	// Set the MSB of the variable.
	cal_temp |= 0x8000;
	
	// Store the value in the plus-side gain calibration registers ADCPGH and ADCPGL.
	//adcptr->CPG = modeptr->plus_gain = modeptr->cal_temp;
	
	// Repeat procedure for the minus-side gain calibration value.
	// Initialize (clear) a 16b variable in RAM.
	cal_temp = 0;

	// Add the following minus-side calibration results CLM0, CLM1, CLM2, CLM3, CLM4, and CLMS to the variable.
	cal_temp += adcptr->CLM0;
	cal_temp += adcptr->CLM1;
	cal_temp += adcptr->CLM2;
	cal_temp += adcptr->CLM3;
	cal_temp += adcptr->CLM4;
	cal_temp += adcptr->CLMS;
	//cal_temp += adcptr->CLMD;
	
	// Divide the variable by two.
	cal_temp >>= 1;
	
	// Set the MSB of the variable.
	cal_temp |= 0x8000;
	
	// Store the value in the minus-side gain calibration registers ADCMGH and ADCMGL.
	//ADCMG = modeptr->minus_gain = modeptr->cal_temp;
	
	return 0;
}

/* start ADC module */
unsigned char adc_start(ADC_MODE *modeptr, unsigned char ch)
{
	ADC_MemMapPtr adcptr = ADC0_BASE_PTR;
	
	modeptr->channel = ch;	
	
	if(ch > 31)
	{
		//print some error message here
		return 1;	//invalid channel
	}
	
	if(modeptr->trigger_src)
	{	//fire PDB trigger here
		//pdb_start();
	}
	else
	{	//need to be careful to set ADC_SC1A here, "ADC_SC1A |= xxx" could not be used here
		//for disabled value for channel setting are all 1s, so this statement will not function
		adcptr->SC1[0] = ADC_SC1_ADCH(ch) | (modeptr->ien << 6);	
	}
	
	return 0;
}

/* Stop ADC module */
void adc_stop(ADC_MODE *modeptr)
{
	ADC_MemMapPtr adcptr = ADC0_BASE_PTR;
	
	modeptr->channel = ADC_CH_DISABLE;	
	
	adcptr->SC1[0] |= ADC_SC1_ADCH(ADC_CH_DISABLE);	
}

/* get adc result once ADC module has been started */
unsigned char get_adc_result(ADC_MODE *modeptr)
{	
	//TODO add support for reading ADCRB
	
	ADC_MemMapPtr adcptr = ADC0_BASE_PTR;
	
	while(! (adcptr->SC1[0] & ADC_SC1_COCO_MASK))
	{
		
	}
	
	modeptr->resulta = (unsigned int)adcptr->R[0];	//this read also clears COCO flag
	
	return 0;
}

/* Store calibration clock result */
void store_calblk(ADC_MODE *modeptr)
{
	ADC_MemMapPtr adcptr = ADC0_BASE_PTR;
	
	modeptr->calblock.clpd = adcptr->CLPD;
	modeptr->calblock.clps = adcptr->CLPS;
	modeptr->calblock.clp4 = adcptr->CLP4;
	modeptr->calblock.clp3 = adcptr->CLP3;
	modeptr->calblock.clp2 = adcptr->CLP2;
	modeptr->calblock.clp1 = adcptr->CLP1;
	modeptr->calblock.clp0 = adcptr->CLP0;	

	modeptr->calblock.clmd = adcptr->CLMD;
	modeptr->calblock.clms = adcptr->CLMS;
	modeptr->calblock.clm4 = adcptr->CLM4;
	modeptr->calblock.clm3 = adcptr->CLM3;
	modeptr->calblock.clm2 = adcptr->CLM2;
	modeptr->calblock.clm1 = adcptr->CLM1;
	modeptr->calblock.clm0 = adcptr->CLM0;	
		
}

/* Dump calibration block */
void dump_calblk(ADC_MODE *modeptr)
{
	//dump calibration registers
	
	modeptr = modeptr;
#ifdef DEBUG	
	print_str("ADC calibration data is as follows.\n");
	print_str("CLPD=0x%x\tCLPS=0x%x\tCLP4=0x%x\tCLP3=0x%x\tCLP2=0x%x\tCLP1=0x%x\tCLP0=0x%x\n",\
	modeptr->calblock.clpd,modeptr->calblock.clps,modeptr->calblock.clp4,modeptr->calblock.clp3,\
	modeptr->calblock.clp2,modeptr->calblock.clp1,modeptr->calblock.clp0);
	print_str("CLMD=0x%x\tCLMS=0x%x\tCLM4=0x%x\tCLM3=0x%x\tCLM2=0x%x\tCLM1=0x%x\tCLM0=0x%x\n",\
	modeptr->calblock.clmd,modeptr->calblock.clms,modeptr->calblock.clm4,modeptr->calblock.clm3,\
	modeptr->calblock.clm2,modeptr->calblock.clm1,modeptr->calblock.clm0);
#endif
	
}





