/*
 * tsi.c
 *
 *  Created on: Apr 4, 2011
 *      Author: B23243
 */
#include "common.h"
#include <tsi.h>

uint16  g16ElectrodeTouch[16] = {0};
uint16  g16ElectrodeBaseline[16] = {0};
uint32  g32DebounceCounter[16] = {DBOUNCE_COUNTS};

/********************************************************************************
 *   tsi_Init: tsi_Init
 * Notes: TSI initialization using MCGIRCLK, TSI able to work on low power modes 
 *    -
 ********************************************************************************/
void tsi_Init(void)
{  

	     //Enable clock gates
	     SIM_SCGC2 |= (SIM_SCGC2_TSI_MASK);

	     MXC_PTBPF4 &= 0x0F; // Select PTB1 as TSICH10 
	     
	     //Configure the number of scans and enable the interrupt
	     TSI0_GENCS |= ((TSI_GENCS_NSCN(3))|TSI_GENCS_STM_MASK|(TSI_GENCS_PS(1)));
	     
	     TSI0_SCANC |= ((TSI_SCANC_EXTCHRG(3))|(TSI_SCANC_REFCHRG(31))|
	                   (TSI_SCANC_DELVOL(7))|(TSI_SCANC_SMOD(0))|(TSI_SCANC_AMPSC(0)));

	     //Enable the channels desired
	     TSI0_PEN |= (TSI_PEN_PEN9_MASK|TSI_PEN_PEN10_MASK);
	     ELECTRODE_ENABLE_REG = ELECTRODE9_EN_MASK|ELECTRODE10_EN_MASK;
		     	 
	     //Enable TSI module
	     TSI0_GENCS |= (TSI_GENCS_TSIEN_MASK); //Enables TSI
	     
	     tsi_calibration(); 	     
	     ENABLE_TSI_ISR;
		

}

/********************************************************************************
 *   TSI_GoToLP: Changes config to go to low power
 * Notes:
 *    -
 ********************************************************************************/
void TSI_GoToLP(void)
{
    DISABLE_TSI;
  
    while(!TSI0_GENCS&TSI_GENCS_EOSF_MASK){}; //Wait for last scan to happen
  
    TSI0_GENCS |= TSI_GENCS_OUTRGF_MASK;   //Clear all pending flags
    TSI0_GENCS |= TSI_GENCS_EOSF_MASK;
    TSI0_STATUS = 0xFFFFFFFF;
  
    TSI0_GENCS |= TSI_GENCS_LPCLKS_MASK;
    
    TSI0_GENCS &= ~(TSI_GENCS_ESOR_MASK);
    LLWU_ME = LLWU_ME_WUME3_MASK;
    ENABLE_TSI;

}

void TSI_GoToActive(void)
{
  DISABLE_TSI;
  
  //while(!TSI0_GENCS&TSI_GENCS_EOSF_MASK){}; //Wait for last scan to happen
  
  TSI0_GENCS |= TSI_GENCS_OUTRGF_MASK;   //Clear all pending flags
  TSI0_GENCS |= TSI_GENCS_EOSF_MASK;
  TSI0_STATUS = 0xFFFFFFFF;
  
  TSI0_GENCS |= (TSI_GENCS_ESOR_MASK);
  
  ENABLE_TSI;  //Enables TSI
}

/********************************************************************************
 *   TSI_ISR: TSI_ISR
 * Notes:
 *    -
 ********************************************************************************/
interrupt void TSI_ISR(void)
{
	//print_str("\nTSI interruption\r\n");
	//print_str("\nvalue %08x\r\n", TSI0_GENCS );
	
	if((TSI0_GENCS & TSI_GENCS_EXTERF_MASK) && ((TSI0_GENCS & TSI_GENCS_OUTRGF_MASK) == 0x0) )
	{
		//print_str("\nElectrode is shorted to VDD or GND \r\n");
				
	}else if(TSI0_GENCS & TSI_GENCS_OUTRGF_MASK)
	{
		if(TSI0_STATUS & 0x00000200)  //Ask for the E1 
		{
			LED1_TOG;
			print_str("\nTSI Electrode 1 touched \r\n");
		}
		if(TSI0_STATUS & 0x00000400) //Ask for the E2 
		{
			LED2_TOG;
			print_str("\nTSI Electrode 2 touched \r\n");
		}
	}

	clear_flags();
}

/********************************************************************************
 *   clear_flags: clear_flags
 * Notes:Clear TSI flags to avoid pending interruptions
 *    -
 ********************************************************************************/
void clear_flags(void)
{
	TSI0_GENCS |= TSI_GENCS_EOSF_MASK;
	TSI0_GENCS |= TSI_GENCS_OUTRGF_MASK;
	TSI0_GENCS |= TSI_GENCS_EXTERF_MASK;
	TSI0_GENCS |= TSI_GENCS_OVRF_MASK;
	TSI0_STATUS |= 0xFFFFFFFF;//clear error flag write one to all possible flags
}

/********************************************************************************
 *   tsi_calibration: tsi_calibration
 * Notes: this function enable TSI threshold for the electrodes 
 *    -
 ********************************************************************************/
void tsi_calibration(void)
{
	uint8 stb=0;
	
	  while(!TSI0_GENCS&TSI_GENCS_EOSF_MASK){};

	/*Spending this time here reading the electrodes value here help to stabilize the signal*/  
	  delay(1500);
   while(stb++ <= 10){
	  g16ElectrodeBaseline[ELECTRODE9] = ELECTRODE9_COUNT;
	  g16ElectrodeTouch[ELECTRODE9] = g16ElectrodeBaseline[ELECTRODE9] + ELECTRODE9_TOUCH;
	  ELECTRODE9_OUTRG = (uint32)g16ElectrodeTouch[ELECTRODE9];

	  g16ElectrodeBaseline[ELECTRODE10] = ELECTRODE10_COUNT;
	  g16ElectrodeTouch[ELECTRODE10] = g16ElectrodeBaseline[ELECTRODE10] + ELECTRODE10_TOUCH;
	  ELECTRODE10_OUTRG = (uint32)g16ElectrodeTouch[ELECTRODE10];
	  
   }
}

/********************************************************************************
 *   delay: delay
 * Notes:
 *    -
 ********************************************************************************/
void delay(uint32 i)
{
  for(i;i;i--)  //delay
  {
  }
}
