/*************************************************************************
 * DISCLAIMER *
 * Services performed by FREESCALE in this matter are performed          *
 * AS IS and without any warranty. CUSTOMER retains the final decision   *
 * relative to the total design and functionality of the end product.    *
 * FREESCALE neither guarantees nor will be held liable by CUSTOMER      *
 * for the success of this project. FREESCALE disclaims all warranties,  *
 * express, implied or statutory including, but not limited to,          *
 * implied warranty of merchantability or fitness for a particular       *
 * purpose on any hardware, software ore advise supplied to the project  *
 * by FREESCALE, and or any product resulting from FREESCALE services.   *
 * In no event shall FREESCALE be liable for incidental or consequential *
 * damages arising out of this agreement. CUSTOMER agrees to hold        *
 * FREESCALE harmless against any and all claims demands or actions      *
 * by anyone on account of any damage, or injury, whether commercial,    *
 * contractual, or tortuous, rising directly or indirectly as a result   *
 * of the advise or assistance supplied CUSTOMER in connection with      *
 * product, services or goods supplied under this Agreement.             *
 *************************************************************************/
/*************************************************************************************************
 * File name   : Bootloader_Bdt.h
 *
 *
 * Description : The BDT definition for USB module 
 * History     :
 * 04/01/2007  : Initial Development
 * 
 *************************************************************************************************/
#pragma options align=packed
#ifndef _BOOTLOADER_BDT_H
#define _BOOTLOADER_BDT_H

#include "derivative.h"
#include "psptypes.h"
#include "Bootloader_Config.h"
#include "Bootloader_Ep0_Handler.h"
#include "Bootloader_Hid.h"

#define CtrlTrf_Data Setup_Pkt

/* Buffer Descriptor Status Register Initialization Parameters */
#define _BDTSTALL      0x04                /*Buffer Stall enable*/
#define _DATA0         0x00                /*DATA0 packet expected next*/
#define _DATA1         0x40                /*DATA1 packet expected next*/
#define _DTS           0x08                /*DTS Mask*/
#define _SIE           0x80                /*SIE owns buffer*/
#define _CPU           0x00                /*CPU owns buffer*/

#define mMCU      0x00
#define mUDATA0   0x88
#define mUDATA1   0xC8




 
typedef union _USB_DEVICE_STATUS
{
	uint_8 _byte;
    struct
    {
		uint_8 Self_Power  :1;
		uint_8 RemoteWakeup:1;        /*[0]Disabled [1]Enabled: */
    }BitCtl;
} USB_DEVICE_STATUS;




typedef union _BD_STAT
{
	uint_8 _byte;
    struct{
    	uint_8 :1;
    	uint_8 :1;
    	uint_8 BDTSTALL:1;            /*Buffer Stall Enable*/
    	uint_8 DTS:1;                 /*Data Toggle Synch Enable*/
    	uint_8 NINC:1;                    /*Address Increment Disable*/
    	uint_8 KEEP:1;                    /*BD Keep Enable*/
    	uint_8 DATA:1;                /*Data Toggle Synch Value*/
    	uint_8 OWN:1;                 /*USB Ownership*/
    }McuCtlBit;
    struct{
    	uint_8    :2;
    	uint_8 PID:4;                 /*Packet Identifier*/
    	uint_8    :2;
    }RecPid;
}   BD_STAT;                              /*Buffer Descriptor Status Register*/

/*
typedef struct _BUFF_DSC
{
        BD_STAT Stat;
        uint_8 Cnt;
        uint_8 Addr;                      //Buffer Address 
} BUFF_DSC;                                  //Buffer Descriptor Table
*/

typedef struct _BUFF_DSC
{
        BD_STAT Stat;
        uint_8 Res1;
        uint_8 CntLSB;
        uint_8 CntMSB;
        uint_8 Addr0LSB;                     //Buffer Address 
        uint_8 Addr1;
        uint_8 Addr2;
        uint_8 Addr3MSB;
}  BUFF_DSC;                                  /*Buffer Descriptor Table*/



typedef struct _BDTMAP
{ 
  BUFF_DSC ep0BDSetup;    //Endpoint 0 BD Setup
  BUFF_DSC ep0BDOut;      //Endpoint 0 BD Out
  BUFF_DSC ep0BDInEven;   //Endpoint 0 BD In Even
  BUFF_DSC ep0BDInOdd;    //Endpoint 0 BD In Odd
  BUFF_DSC ep1BDOut;       //Endpoint 1 BD IN or OUT
  BUFF_DSC ep1BDOut1;      //Endpoint 1 BD IN or OUT
  BUFF_DSC ep1BDIn;       //Endpoint 1 BD IN or OUT
  BUFF_DSC ep1BDIn1;      //Endpoint 1 BD IN or OUT

}BDTMAP; 


#endif 