/*************************************************************************
 * DISCLAIMER *
 * Services performed by FREESCALE in this matter are performed          *
 * AS IS and without any warranty. CUSTOMER retains the final decision   *
 * relative to the total design and functionality of the end product.    *
 * FREESCALE neither guarantees nor will be held liable by CUSTOMER      *
 * for the success of this project. FREESCALE disclaims all warranties,  *
 * express, implied or statutory including, but not limited to,          *
 * implied warranty of merchantability or fitness for a particular       *
 * purpose on any hardware, software ore advise supplied to the project  *
 * by FREESCALE, and or any product resulting from FREESCALE services.   *
 * In no event shall FREESCALE be liable for incidental or consequential *
 * damages arising out of this agreement. CUSTOMER agrees to hold        *
 * FREESCALE harmless against any and all claims demands or actions      *
 * by anyone on account of any damage, or injury, whether commercial,    *
 * contractual, or tortuous, rising directly or indirectly as a result   *
 * of the advise or assistance supplied CUSTOMER in connection with      *
 * product, services or goods supplied under this Agreement.             *
 *************************************************************************/
 /************************************************************************
 * File name   : Bootloader_drv.c
 * Description : This file provides the USB driver, all USB events will be 
 *               precessed
 *               
 *
 * History     :
 * 04/01/2007  : Initial Development
 * 
 *************************************************************************/
#pragma options align=packed
#include "derivative.h"
#include "Bootloader_Config.h"
#include "Bootloader_Drv.h"
#include "Bootloader_Bdt.h"
#include "Bootloader_Ep0_Handler.h"
#include "Bootloader_Hid.h"
#include "Bootloader_Bdt.h"


/*Global variable definition*/

uint_8 Usb_Device_State;          /* device states*/
USB_DEVICE_STATUS Usb_Stat;     /* global flags*/
uint_8 Usb_Active_Cfg;            /* value of current configuration*/
uint_8 Usb_Alt_Intf[MAX_NUM_INTF]; /* array to keep track of the current alternate*/
                                /* setting for each interface ID*/


extern BDTMAP Bdtmap;
extern CTRL_TRANSFER_DATA  CtrlTrf_IN_Data;
extern CTRL_TRANSFER_SETUP Setup_Pkt;
extern uint_8 Hid_Report_In[];

/******************************************************************************
 * Function:        void Initialize_USBModule(void)
 * Input:           None
 * Output:          None
 * Overview:        initialize the USB module
 *                  enable regulator, PHY, pull-up resistor, BDT initialization
 *
 * Note:            None
 *****************************************************************************/
/******************************************************************************
 * Function:        void Initialize_USBModule(void)
 * Input:           None
 * Output:          None
 * Overview:        initialize the USB module
 *                  enable regulator, PHY, pull-up resistor, BDT initialization
 *
 * Note:            None
 *****************************************************************************/
void  Initialize_USBModule()
{

	volatile uint_8 tmp;
	/* Enable clock source for USB module */
	SIM_SCGC6 |= SIM_SCGC6_USBOTG_MASK;
	SIM_SOPT1 |= SIM_SOPT1_REGE_MASK;
	
	
	Usb_Device_State = POWERED_STATE;
  #if (SELF_POWER)
    Usb_Stat._byte = 0x01;                /*SelfPower*/
  #else
    Usb_Stat._byte = 0x00;                /*Notice: Bus powered*/
  #endif
  Usb_Active_Cfg = 0x00;


  USB0_USBTRC0 |= USB_USBTRC0_USBRESET_MASK;

  while(USB0_USBTRC0 & USB_USBTRC0_USBRESET_MASK){};
  

//  USB_CTRL = INT_STAT_USB_RST_MASK;     // clear USB reset flag



  /*Below is initialize the BDT*/

  Clear_Mem((uint_8*)&Bdtmap,sizeof(Bdtmap));    			// disable all endpoints except EP0 */

  Bdtmap.ep0BDSetup.Stat._byte = _SIE|_DATA0|_DTS;  		// endpoint 0 SETUP/OUT initialization
//  Bdtmap.ep0BDSetup.CntMSB = 0; 							// done in Clear_Mem
  Bdtmap.ep0BDSetup.CntLSB = 8;              
//  Bdtmap.ep0BDSetup.Addr3MSB = (uint_8) ((uint_32)&Setup_Pkt >> 24);  // can skip only located at 0x00 XX XX XX 
  Bdtmap.ep0BDSetup.Addr2 = (uint_8) ((uint_32)&Setup_Pkt >> 16);      	//  
//  Bdtmap.ep0BDSetup.Addr1 = (uint_8) ((uint_32)&Setup_Pkt >> 8);      // can skip only located at 0xXX XX 00 XX 
  Bdtmap.ep0BDSetup.Addr0LSB = (uint_8) ((uint_32)&Setup_Pkt >> 0);     //


//  Bdtmap.ep0BDOut.Stat._byte = _SIE|_DATA1|_DTS;  /* endpoint 0 OUT initialization*/
//  Bdtmap.ep0BDOut.CntMSB=0x00;
//  Bdtmap.ep0BDOut.CntLSB = EP0OUT_BUFF_SIZE;              
//  Bdtmap.ep0BDOut.Addr3MSB = (uint_8) ((uint_32)&CtrlTrf_Data >> 24);      //
//  Bdtmap.ep0BDOut.Addr2 = (uint_8) ((uint_32)&CtrlTrf_Data >> 16);      //
//  Bdtmap.ep0BDOut.Addr1 = (uint_8) ((uint_32)&CtrlTrf_Data >> 8);      //
//  Bdtmap.ep0BDOut.Addr0LSB = (uint_8) ((uint_32)&CtrlTrf_Data >> 0);      //


//  Bdtmap.ep0BDInEven.Stat._byte = _CPU;                /* endpoint 0 IN buffer initialization*/
//  Bdtmap.ep0BDInEven.CntMSB = 0x00; 
//  Bdtmap.ep0BDInEven.CntLSB = 0x00;                      
//  Bdtmap.ep0BDInEven.Addr3MSB = (uint_8) ((uint_32)&CtrlTrf_IN_Data >> 24);    // can skip only located at 0x00 XX XX XX  
  Bdtmap.ep0BDInEven.Addr2 = (uint_8) ((uint_32)&CtrlTrf_IN_Data >> 16);	      
//  Bdtmap.ep0BDInEven.Addr1 = (uint_8) ((uint_32)&CtrlTrf_IN_Data >> 8); 		// can skip only located at 0xXX XX 00 XX          
  Bdtmap.ep0BDInEven.Addr0LSB = (uint_8) ((uint_32)&CtrlTrf_IN_Data >> 0);


  USB0_BDTPAGE1 = (uint_8) ((uint_32)&Bdtmap >> 8);      // can skip only located at 0xXX XX 00 XX 
  USB0_BDTPAGE2 = (uint_8) ((uint_32)&Bdtmap >> 16);
  USB0_BDTPAGE3 = (uint_8) ((uint_32)&Bdtmap >> 24);	// can skip only located at 0x00 XX 00 XX 


  USB0_USBCTRL = 0 ;              // system clock
//  USB0_CONTROL |= USB_CONTROL_DPPULLUPNONOTG_MASK;
  USB0_ENDPT0 = USB_ENDPT_EPHSHK_MASK | USB_ENDPT_EPTXEN_MASK | USB_ENDPT_EPRXEN_MASK;    /*enable endpoint 0*/
//  USB0_CTL = USB_CTL_USBENSOFEN_MASK;                   // USB module enable
  USB0_USBTRC0 = 0;
 // USBTRC0 = USBTRC0_USBPU_MASK | USBTRC0_USBVREN_MASK;    //0x44;// enable USB vreg and pullup resistor

}


/******************************************************************************
 * Function:        void USB_Bus_Reset_Handler(void)
 *
 * Input:           None
 * Output:          None
 *
 * Overview:        Once a USB bus reset is received from the host, this
 *                  function should be called. It resets the device address to
 *                  zero, disables all non-EP0 endpoints, initializes EP0 to
 *                  be ready for default communication, clears all USB
 *                  interrupt flags, unmasks applicable USB interrupts, and
 *                  reinitializes internal state-machine.
 *****************************************************************************/
void USB_Bus_Reset_Handler(void)
{
	//    Usb_Stat.BitCtl.RemoteWakeup = 0;       /* default status flag to disable*/
    USB0_ADDR = 0x00;                    /* reset to default address*/
    Usb_Active_Cfg = 0;                    
    Usb_Device_State = DEFAULT_STATE;
    Clear_Mem((uint_8*)&Bdtmap.ep1BDIn,1);    /* disable all endpoints except EP0 */
    if (USB0_ENDPT0 !=  (USB_ENDPT_EPTXEN_MASK | USB_ENDPT_EPRXEN_MASK | USB_ENDPT_EPHSHK_MASK))
   {
		USB0_ENDPT0 = USB_ENDPT_EPTXEN_MASK | USB_ENDPT_EPRXEN_MASK | USB_ENDPT_EPHSHK_MASK;	
	}

	USB_Prepare_Next_Trf();   /*prepare to receive the setup packet*/

 
    while(USB0_ISTAT & USB_ISTAT_TOKDNE_MASK)
    {/* Flush any pending transactions because 4 status buffer*/
    	USB0_ISTAT = USB_ISTAT_TOKDNE_MASK;              
    }
    USB0_ENDPT0 = 0x0D;                  // enable endpoint
    USB0_INTEN = 0x00;                 // disable USB interrupt
    USB0_ERREN = 0x00;                 // disable all error interrupt            
    USB0_ISTAT = 0xFF;
    USB0_CTL = USB_CTL_USBENSOFEN_MASK | USB_CTL_ODDRST_MASK;
 
}


/******************************************************************************
 * Function:        void Check_USBBus_Status(void)
 * Input:           None
 * Output:          None
 * Overview:        This function can be used to detect if the USB bus has attached
 *                  on USB bus, we can use a GPIO, or KBI interrupt, it is disable here
 *
 *****************************************************************************/

void Check_USBBus_Status(void)
{
    #if (SELF_POWER)
	if (PTA_D & 0x02)
	{  
    #endif
        if((USB0_CTL & USB_CTL_USBENSOFEN_MASK)==0)          // device module off ? 
         {

//            USB0_OTGCTL |=  USB_OTGCTL_DPHIGH_MASK | USB_OTGCTL_OTGEN_MASK;
            USB0_CTL = USB_CTL_USBENSOFEN_MASK;       // enable module 
//            USB0_CONTROL |= USB_CONTROL_DPPULLUPNONOTG_MASK;
              USB0_CONTROL = USB_CONTROL_DPPULLUPNONOTG_MASK;

            Usb_Device_State = ATTACHED_STATE;   
 
         }
  
   #if (SELF_POWER)
     }
    else
    {
        USB0_CTL = 0x00;                        /* disable module */
        Usb_Device_State = POWERED_STATE;      
    }
   #endif
   

    if(Usb_Device_State == USB_ENTER_SUSPEND)
    {
        USB_Suspend();
    }
}



/******************************************************************************
 * Function:        void USB_Soft_Detach(void)
 * Input:           None
 * Output:          None
 * Overview:        USB_Soft_Detach disconnects the device from USB bus. 
 *                  This is done by stop supplying Vusb voltage to
 *                  pull-up resistor. The pull-down resistors on the host
 *                  side will pull both differential signal lines low and
 *                  the host registers the event as a disconnect.
 *
 *                  Since the USB cable is not physically disconnected, the
 *                  power supply through the cable can still be sensed by
 *                  the device. The next time Check_USBBus_Status() function
 *                  is called, it will reconnect the device back to the bus.
 *****************************************************************************/
void USB_Soft_Detach(void)
{
   USB0_CTL = 0x00;                             /* disable module */
   USB0_INTEN = 0x00;                          /* mask USB interrupt*/
   Usb_Device_State = POWERED_STATE;      
   return;
}


/******************************************************************************
 * Function:        void USB_Suspend(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        After 3ms of USB bus idle status, the SLEEPF will be set,
 *                  The firmware should enable the resume function in 7ms before 
 *                  the USB enters into suspend state.       
 * Note:            None
 *****************************************************************************/
void USB_Suspend(void)
{

    //INTENB_RESUME = 1;   
    USB0_USBTRC0 |= USB_USBTRC0_USBRESMEN_MASK;
                     
    USB0_ISTAT  = USB_ISTAT_SLEEP_MASK;                    
   
    Usb_Device_State = USB_SUSPEND;
       
//    USBCTL0_USBRESMEN = 0;  
    
 	return;
}

/******************************************************************************
 * Function:        void USB_WakeFrom_Suspend(void)
 * Input:           None
 * Output:          None
 * Overview:        
 *****************************************************************************/
void USB_WakeFrom_Suspend(void)
{
    USB0_ISTAT |= USB_ISTAT_RESUME_MASK;  /*write 1 to clear*/
    USB0_INTEN &= ~USB_USBTRC0_USB_RESUME_INT_MASK;
    USB0_CTL &= ~USB_CTL_TXSUSPENDTOKENBUSY_MASK;	    /*enable the transmit or receive of packet */

}

         
/******************************************************************************
 * Function:        void USB_SOF_Handler(void)
 * Input:           None
 *
 * Output:          None
 * Overview:        The USB host sends out a SOF packet to full-speed devices
 *                  every 1 ms. This interrupt may be useful for isochronous
 *                  pipes. 
 *****************************************************************************/
//void USB_SOF_Handler(void)
//{
    /*can add some code if isochronous transfer*/
    
//    INTSTAT_SOFTOKF = 1;   /*clear the interrupt flag*/
//}

/******************************************************************************
 * Function:        void USB_Stall_Handler(void)
 * Input:           None
 * Output:          None
 * Overview:        The STALLIF is set anytime the SIE sends out a STALL
 *                  packet regardless of which endpoint causes it.
 *                  A Setup transaction overrides the STALL function. A stalled
 *                  endpoint stops stalling once it receives a setup packet.
 *
 *                  There are a few reasons for an endpoint to be stalled.
 *                  1. When a non-supported USB request is received.
 *                     Example: GET_DESCRIPTOR(DEVICE_QUALIFIER)
 *                  2. When an endpoint is currently halted.
 *                  3. When the device class specifies that an endpoint must
 *                     stall in response to a specific event.
 *
 *****************************************************************************/
void USB_Stall_Handler(void)
{
    if((USB0_STAT & 0xF0)== 0x00)
    {
      if(USB0_ENDPT0 |= USB_ENDPT_EPSTALL_MASK)
      {
    	  USB0_ENDPT0 &= ~USB_ENDPT_EPSTALL_MASK;
      }    
      /*Notice if is generated by BDTSTALL, the firmware should notice the IN or OUT DIR*/
      /*we only set it for OUT direction in this demo*/
      USB_Prepare_Next_Trf();     
    }
    USB0_ISTAT = USB_ISTAT_STALL_MASK;
}



/******************************************************************************
 * Function:        void USB_Error_Handler(void)
 * Input:           None
 * Output:          None
 * Overview:        You can check error register to see which error occured
 * Note:            None
 *****************************************************************************/
void USB_Error_Handler(void)
{
 //   USB0_ERRSTAT |= INT_STAT_ERROR = 1;
    /*add code to check the error source */

    if((USB0_ERRSTAT |= USB_ERRSTAT_BTSERR_MASK) && ((USB0_STAT & 0xF0)==0)) /*Bdt buffer overflow*/
      {
        USB_Prepare_Next_Trf();  
      }
      
    /*check the other error at here */
      
    USB0_ERRSTAT = 0xFF; /*clear all errors*/
    return;
}






/******************************************************************************
 * Function:        void USB_Buf_Rdy(buffer_dsc)
 *
 * PreCondition:    IN endpoint: Buffer is control by MCU.
 *                  OUT endpoint: Buffer is conrolled by SIE.
 *
 * Input:           byte buffer_dsc: one buffer descriptor in Bdtmap
 * Output:          None
 *
 * Overview:        This function turns the buffer ownership to SIE,
 *                  and toggles the DTS bit for synchronization.
 * Note:            This function should not be called by Endpoint 5 or 6
 *                  because they are pingpong buffer, DATA0/1 do not need 
 *                  to be switched
 *****************************************************************************/
void USB_Buf_Rdy(BUFF_DSC *buffer_dsc)
{                                                        
    buffer_dsc->Stat.McuCtlBit.DATA = ~ buffer_dsc->Stat.McuCtlBit.DATA; /* Toggle DTS bit*/
    buffer_dsc->Stat._byte &= _DATA1;       
    buffer_dsc->Stat._byte |= _SIE|_DTS;                              /*give the ownership to SIE*/
}


/******************************************************************************
 * Function:        void Clear_Mem(byte* startAdr,byte count)
 * Input:          
 * Output:          None
 * Overview:        None
 *****************************************************************************/
void Clear_Mem(uint_8* startAdr,uint_32 count)
{
	uint_32 i;
        
    for(i=0; i < count; i++)
    {
       *(startAdr + i) = 0x00;
    }
    
    return;
}



/******************************************************************************
 * Function:       void interrupt  USB_ISR()
 * Input:          
 * Output:          None
 * Overview:        The USB stat interrupt service routine
 * Note:            None
 *****************************************************************************/


void USB_Bootloader()
{
 
   if(USB0_ISTAT & USB_ISTAT_USBRST_MASK)
    {
      USB_Bus_Reset_Handler();
    }

 
   if(USB0_ISTAT & USB_ISTAT_SLEEP_MASK)  

    {
      Usb_Device_State = USB_ENTER_SUSPEND; 
      USB0_ISTAT = USB_ISTAT_SLEEP_MASK; 
    }

/*       
  if(USBCTL0_LPRESF) 
    {
      USBTRC0_USBRESMEN = 0;      // disable wakeup from suspend
    }
*/
  
     
  if(USB0_ISTAT & USB_ISTAT_RESUME_MASK)
	     USB_WakeFrom_Suspend();
 
  if(USB0_ISTAT & USB_ISTAT_STALL_MASK)
      USB_Stall_Handler();

  
  if(USB0_ISTAT & USB_ISTAT_ERROR_MASK)
        USB_Error_Handler(); 
   
  if(USB0_ISTAT & USB_ISTAT_TOKDNE_MASK)
    {
     if (Usb_Device_State>= DEFAULT_STATE)
     {
    	 USB_Transaction_Handler();
    	 USB0_ISTAT =  USB_ISTAT_TOKDNE_MASK;     /*write 1 to clear*/
      }
    }

 
}



/******************************************************************************
 * Function:        unsigned char USB_WakeUp()
 * Input:          
 * Output:          0: noise, re-entern into sleep
 *                  1: Bus wake-up
 *                  2: remote wake up
 * Overview:        The USB sleep and wake up
 * Note:            None
 *****************************************************************************/
 unsigned char USB_WakeUp(void )
 {
    _Stop;                                /*MCU enter into stop3*/
    Usb_Device_State = CONFIGURED_STATE;

/*    
    if((! USBCTL0_LPRESF) )     //Is the KBI interrupt bring the MCU out of STOP3
    {

       if(Usb_Stat.BitCtl.RemoteWakeup == 1) //Is remote wakeup supported
         return 2;                           //will enable remote wakeup
       else
         return 0;
    }                                        //tell MCU it should enter STOP3 again
    else 
    {                                        //add delay to filter the noise 
        delay_count = 50;               
        do
         {
           delay_count--;
           __RESET_WATCHDOG();
         }while(delay_count);        
        
        if(INTSTAT_RESUMEF)                  //The resume signal has engouh time to set RESUMEF bit
        {
            return 1;                        //USB resume
        }
        else
            return 0;                        //resume is caused by noise
        
     }
*/
     
}      
