/*************************************************************************
 * DISCLAIMER                                                            *
 * Services performed by FREESCALE in this matter are performed          *
 * AS IS and without any warranty. CUSTOMER retains the final decision   *
 * relative to the total design and functionality of the end product.    *
 * FREESCALE neither guarantees nor will be held liable by CUSTOMER      *
 * for the success of this project. FREESCALE disclaims all warranties,  *
 * express, implied or statutory including, but not limited to,          *
 * implied warranty of merchantability or fitness for a particular       *
 * purpose on any hardware, software ore advise supplied to the project  *
 * by FREESCALE, and or any product resulting from FREESCALE services.   *
 * In no event shall FREESCALE be liable for incidental or consequential *
 * damages arising out of this agreement. CUSTOMER agrees to hold        *
 * FREESCALE harmless against any and all claims demands or actions      *
 * by anyone on account of any damage, or injury, whether commercial,    *
 * contractual, or tortuous, rising directly or indirectly as a result   *
 * of the advise or assistance supplied CUSTOMER in connection with      *
 * product, services or goods supplied under this Agreement.             *
 *************************************************************************/
/*************************************************************************************************
 * File name   : Usb_Setup_Handler.h
 *
 * Description : This file defines some structure for USB standard request 
 *
 * History     :
 * 04/01/2007  : Initial Development
 * 
 ************************s*************************************************************************/
#ifndef BOOTLOADER_EP0_HANDLER_H
#define BOOTLOADER_EP0_HANDLER_H

#pragma options align=packed
#include "derivative.h"
#include "psptypes.h"

/******************************************************************************
 * Standard Request Codes
 * USB 2.0 Spec Ref Table 9-4
 *****************************************************************************/
#define GET_STATUS  0
#define CLR_FEATURE 1
#define SET_FEATURE 3
#define SET_ADR     5
#define GET_DSC     6
#define SET_DSC     7
#define GET_CFG     8
#define SET_CFG     9
#define GET_INTF    10
#define SET_INTF    11
#define SYNCH_FRAME 12

/* Standard Feature Selectors */
#define DEVICE_REMOTE_WAKEUP    0x00
#define ENDPOINT_HALT           0x00

/* USB PID: Token Types - See chapter 8 in the USB specification*/
#define SETUP_TOKEN         0b00001101
#define OUT_TOKEN           0b00000001
#define IN_TOKEN            0b00001001

/* bmRequestType Definitions */
#define HOST_TO_DEV         0
#define DEV_TO_HOST         1

#define STANDARD            0x00
#define CLASS               0x01
#define VENDOR              0x02

#define RCPT_DEV            0
#define RCPT_INTF           1
#define RCPT_EP             2
#define RCPT_OTH            3

/*Control transfer state machine*/
#define WAIT_SETUP              0
#define CTL_TRF_DATA_TX         1
#define CTL_TRF_DATA_RX         2

/******************************************************************************
 * CTRL_TRANSFER_SETUP:
 *
 * There are 8 bytes for each setup packet. 
 * The buffer size can be  8, 16, 32, or 64, it is determine by EP0_BUFF_SIZE in
 * this siftware.
 *
 * These 8 bytes are defined to be directly addressable to improve access speed
 * and reduce code size.
 *
 *****************************************************************************/
typedef union _CTRL_TRANSFR_SETUP
{
    struct
    {
    	uint_8 _byte[EP0OUT_BUFF_SIZE];			// setup and EP0 share the same buffer
    };
    
    /* Standard Device Requests */
    struct
    {
    	uint_8 bmRequestType;
    	uint_8 bRequest;    
        uint_16 wValue;
        uint_16 wIndex;
        uint_16 wLength;
    }StdCtl;
    
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_16 W_Value;
    	uint_16 W_Index;
    	uint_16 W_Length;
    }CtlPara;

    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 b_ValueLSB;
    	uint_8 b_ValueMSB;
    	uint_8 b_IndexLSB;
    	uint_8 b_IndexMSB;
    	uint_8 b_LengthLSB;
    	uint_8 b_LengthMSB;
    }CtlParaByte;
    
    
    struct
    {
    	uint_8 Recipient:5;           /*Device,Interface,Endpoint,Other*/
    	uint_8 RequestType:2;         /*Standard,Class,Vendor,Reserved*/
    	uint_8 DataDir:1;             /*Host-to-device,Device-to-host*/
    	uint_8 :8;
    	uint_8 bFeature;                  /*DEVICE_REMOTE_WAKEUP,ENDPOINT_HALT*/
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    }CtlReqT;
    
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 bDscIndex;                 /*For Configuration and String DSC Only*/
    	uint_8 bDscType;                  /*Device,Configuration,String*/
    	uint_16 wLangID;                   /*Language ID*/
        uint_8 :8;
        uint_8 :8;
    }ReqWval;
    
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 bDevADR;                   /*Device Address 0-127*/
    	uint_8 bDevADRH;                  /*Must equal zero*/
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    }CtlAdd;
    
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 bCfgValue;                 /*Configuration Value 0-255*/
    	uint_8 bCfgRSD;                   /*Must equal zero (Reserved)*/
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    }CtrCfg;
    
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 bAltID;                    /*Alternate Setting Value 0-255*/
    	uint_8 bAltID_H;                  /*Must equal zero*/
    	uint_8 bIntfID;                   /*Interface Number Value 0-255*/
    	uint_8 bIntfID_H;                 /*Must equal zero*/
    	uint_8 :8;
    	uint_8 :8;
    }CtlIntf;
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 bEPID;                     /*Endpoint ID (Number & Direction)*/
    	uint_8 bEPID_H;                   /*Must equal zero*/
    	uint_8 :8;
    	uint_8 :8;
    }CtlEp;
 
    struct
    {
    	uint_8 _byte0;
    	uint_8 _byte1;
    	uint_8 _byte2;
    	uint_8 _byte3;
    	uint_8 _byte4;
    	uint_8 _byte5;
    	uint_8 _byte6;
    	uint_8 _byte7;
    }EachByte;
  
    
    struct
    {
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 EPNum:4;               /*Endpoint Number 0-15*/
    	uint_8 :3;
    	uint_8 EPDir:1;               /*Endpoint Direction: 0-OUT, 1-IN*/
    	uint_8 :8;
    	uint_8 :8;
    	uint_8 :8;
    }EpND;
    
} CTRL_TRANSFER_SETUP;

/******************************************************************************
 * CTRL_TRANSFER_DATA:
 * The size can be 8, 16, 32, or 64. It is specified by the value of EP0_BUFF_SIZE 
 *
 * These 8 bytes are defined to be directly addressable to improve access speed
 * and reduce code size.
 *****************************************************************************/
typedef union _CTRL_TRANSFER_DATA
{
    /*Array for indirect addressing*/
    struct
    {
    	uint_8 _byte[SETUP_BUFF_SIZE];
    }ByteArray;
    
    /*First 8-byte direct addressing */
    struct
    {
    	uint_8 _byte0;
    	uint_8 _byte1;
    	uint_8 _byte2;
    	uint_8 _byte3;
    	uint_8 _byte4;
    	uint_8 _byte5;
    	uint_8 _byte6;
    	uint_8 _byte7;
    }EachByte;
    
    struct
    {
    	uint_16 _word0;
    	uint_16 _word1;
    	uint_16 _word2;
    	uint_16 _word3;
    }Eachword;

} CTRL_TRANSFER_DATA;

/* Local functions definition */
void USB_CtrlTrf_SetupStage_Processing(void);
void USB_CtrlEP_SetupStage_Complete(void);

void USB_CtrlTrf_OUT_Handler(void);
void USB_CtrlTrf_IN_Handler(void);

void USB_CtrlTrf_RxData(void);
void USB_CtrlTrf_TxData(void);

void USB_StdReq_Handler(void);
void USB_ClassReq_Handler(void);
void USB_StdGetDsc_Handler(void);
void USB_StdSetCfg_Handler(void);
void USB_StdGetStatus_Handler(void);
void USB_StdFeatureReq_Handler(void);


/* Global functions definition  */
void USB_Transaction_Handler(void);
void USB_Prepare_Next_Trf(void);



#endif
