/*************************************************************************
 * DISCLAIMER *
 * Services performed by FREESCALE in this matter are performed          *
 * AS IS and without any warranty. CUSTOMER retains the final decision   *
 * relative to the total design and functionality of the end product.    *
 * FREESCALE neither guarantees nor will be held liable by CUSTOMER      *
 * for the success of this project. FREESCALE disclaims all warranties,  *
 * express, implied or statutory including, but not limited to,          *
 * implied warranty of merchantability or fitness for a particular       *
 * purpose on any hardware, software ore advise supplied to the project  *
 * by FREESCALE, and or any product resulting from FREESCALE services.   *
 * In no event shall FREESCALE be liable for incidental or consequential *
 * damages arising out of this agreement. CUSTOMER agrees to hold        *
 * FREESCALE harmless against any and all claims demands or actions      *
 * by anyone on account of any damage, or injury, whether commercial,    *
 * contractual, or tortuous, rising directly or indirectly as a result   *
 * of the advise or assistance supplied CUSTOMER in connection with      *
 * product, services or goods supplied under this Agreement.             *
 *************************************************************************/
/*************************************************************************************************
 * File name   : Bootloader_tasks.c
 *
 * Description : This file processes the mouse report 
 *               
 *
 * History     :
 * 10/09/2010  : Derek Lau    ; Initial Development
 * 
 *************************************************************************************************/
#pragma options align=packed
#include "derivative.h"
#include "Bootloader_Bdt.h"
#include "Bootloader_drv.h"
#include "bootloader_tasks.h"



#define BL_COMMAND_ERASE      0x01
#define BL_COMMAND_PROGRAM    0x11
#define BL_COMMAND_GET_STATUS 0x81

#define Flash_OK          	0x00
#define Flash_ERROR     	0xFF

extern uint_8 Flash_SectorErase(uint_32 FlashPtr);
extern uint_8 Flash_ByteProgram(uint_32 FlashStartAdd,uint_32 *DataSrcPtr,uint_32 NumberOfBytes);
extern uint_8 Flash_ByteVerify(uint_32 FlashStartAdd,uint_8 *DataSrcPtr,uint_32 NumberOfBytes);

extern BDTMAP Bdtmap;
extern CTRL_TRANSFER_DATA  CtrlTrf_IN_Data;
extern CTRL_TRANSFER_SETUP Setup_Pkt;
extern uint_8 Hid_Report_In[HID_INT_IN_EP_SIZE];

/******************************************************************************
 * Function:        void HID_Rec_Data(void)
 * Input:           None
 * Output:          None
 * Overview:        Per HID class std. 1.1, it supports bidirectional communication
 *                  this function is used to receive the data transferred by host
 *****************************************************************************/
void HID_Rec_Data(void)
{
  uint_8 result;
  
  CommandBuffer = (BootloaderCommand*) &Setup_Pkt;
  
  if (CommandBuffer->C.bCommand == BL_COMMAND_PROGRAM){
      result=Flash_ByteProgram(CommandBuffer->Program.dwFlashAddress,(uint_32*)&CommandBuffer->Program.Data,(uint_32)CommandBuffer->Program.wLength);
      if (result==Flash_OK)
       {
          result=Flash_ByteVerify(CommandBuffer->Program.dwFlashAddress,(uint_8*)&CommandBuffer->Program.Data,(uint_32)CommandBuffer->Program.wLength);
       }
  } 
  
  else if (CommandBuffer->C.bCommand == BL_COMMAND_ERASE){
      result=Flash_SectorErase(CommandBuffer->Erase.dwFlashAddress);
  }
  

/*  
  if(!HID_BD_OUT.Stat.McuCtlBit.OWN)
  {
      USB_Buf_Rdy((struct _BUFF_DSC *)&HID_BD_OUT);
  }
*/
  
  Hid_Report_In[0] = 0x5a;
  Hid_Report_In[1] = result;

   HID_BD_IN.CntLSB = HID_INT_IN_EP_SIZE;
   USB_Buf_Rdy((struct _BUFF_DSC *)&HID_BD_IN);  
   
}



