/*
 * File:	start.c
 * Purpose:	Kinetis start up routines. 
 *
 * Notes:		
 */

#include "start.h"
#include "common.h"
#include "wdog.h"
#include "sysinit.h"
#include "vectors.h"
#include "usb_icp.h"
#include "flash.h"
#include "SSD_FTFL.h"

/********************************************************************/
/*!
 * \brief   Kinetis Start
 * \return  None
 *
 * This function calls all of the needed starup routines and then 
 * branches to the main process.
 */
#define GPIO_PIN_MASK            0x1Fu                          // usb_hid_icp
#define GPIO_PIN(x)              (((1)<<(x & GPIO_PIN_MASK)))   // usb_hid_icp


#pragma optimize=medium                                         // usb_hid_icp
//void start(void)@".usb_hid_icp"
void start(void)
{ 
  /* Disable the watchdog timer */
  
  uint32 *User_Vector;
  unsigned short i;	
  UINT8  *data;
  UINT32   CheckSum = 0;
  UINT32   AppCheckSum;

  AppCheckSum = ((UINT32*)APPCHECKSUM32)[0];

  DisableInterrupts;
  /* Write 0xC520 to the unlock register */
   WDOG_UNLOCK = 0xC520;
	
  /* Followed by 0xD928 to complete the unlock */
  WDOG_UNLOCK = 0xD928;
  WDOG_STCTRLH &= ~WDOG_STCTRLH_WDOGEN_MASK;

#if (defined(PFLASH_64K))
  #define SET_SP       "MOV r0,#0x1000"    // 0x4000 - User Program Vector Table address
   FTFL_FPROT3 = 0xFD;          // 4K Flash protected
  
#elif (defined(PFLASH_128K))
  #define SET_SP       "MOV r0,#0x1000"    // 0x4000 - User Program Vector Table address
   FTFL_FPROT3 = 0xFE;          // 4K Flash protected
  
#elif (defined(PFLASH_256K))
  #define SET_SP       "MOV r0,#0x2000"    // 0x4000 - User Program Vector Table address
  FTFL_FPROT3 = 0xFE;          // 8K Flash protected
  
#elif (defined(PFLASH_512K))
  #define SET_SP       "MOV r0,#0x4000"    // 0x4000 - User Program Vector Table address
  FTFL_FPROT3 = 0xFE;          // 16K Flash protected 
#else
  #error "No FPlash size is defined, eg: PFLASH 512K"
#endif


#if (defined(CPU_MK40N512VMD100))  //Look at status of SW6 on PTC5   
  SIM_SCGC5 = SIM_SCGC5_PORTC_MASK;   // enable clock to port C
  PORTC_PCR5=(PORT_PCR_MUX(1)|PORT_PCR_PE_MASK|PORT_PCR_PS_MASK);   // sw6 pullup


//  WDOG_STCTRLH &= ~WDOG_STCTRLH_WDOGEN_MASK;  
  
  for (i=0; i<0xfff; i++){    
    
    if((GPIOC_PDIR & GPIO_PDIR_PDI(GPIO_PIN(5)))!=0) //if not pressed
    {
     User_Vector = (uint32*) USER_ADDR+4;
      if (*User_Vector != 0xFFFFFFFF){
  
      PORTC_PCR5=0;
      SIM_SCGC5 = 0;

      data = (UINT8*)USER_ADDR;
      for (i=0; i< FTFL_SECTOR_SIZE;i++)
      {
        CheckSum += *data++;
      }

      CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[0];
      CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[1];
      CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[2];
      CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[3];
      
      if (CheckSum != AppCheckSum)
      {
        return;
      }
      
#elif (defined(CPU_MK60N512VMD100))
      
  SIM_SCGC5 = SIM_SCGC5_PORTA_MASK;
  PORTA_PCR19=(PORT_PCR_MUX(1)|PORT_PCR_PE_MASK|PORT_PCR_PS_MASK);
  
    for (i=0; i<0xfff; i++){  
     if((GPIOA_PDIR & GPIO_PDIR_PDI(GPIO_PIN(19)))!=0) //if not pressed
    {
     User_Vector = (uint32*) USER_ADDR+4;
      if (*User_Vector != 0xFFFFFFFF)
      {
        PORTA_PCR19 = 0;
        SIM_SCGC5 = 0;

        data = (UINT8*)USER_ADDR;
        for (i=0; i< FTFL_SECTOR_SIZE;i++)
        {
          CheckSum += *data++;
        }

        CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[0];
        CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[1];
        CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[2];
        CheckSum = CheckSum-((UINT8*) APPCHECKSUM8)[3];
        
        if (CheckSum != AppCheckSum)
        {
          return;
        }
     
 //       PORTA_PCR19 = 0;
 //       SIM_SCGC5 = 0;
        
        
#endif
        asm (SET_SP);                 // User Program Vector Table address
        asm  ("LDR SP,[r0]");         // setup SP
        asm ("ADD r0,r0,#4");
        asm  ("LDR r0,[r0]");         // load user program __startup address
        asm  ("BX  r0") ;             // jump to user program
       
      }
    } 
  }
  /* Write 0xC520 to the unlock register */	
  /* followed by 0xD928 to complete the unlock */
  WDOG_UNLOCK = 0xC520;
  WDOG_UNLOCK = 0xD928;
  WDOG_STCTRLH &= ~WDOG_STCTRLH_WDOGEN_MASK;
	/* Copy any vector or data sections that need to be in RAM */
	common_startup();

	/* Perform processor initialization */
	sysinit();

//        Init_Flash_Driver();
	/* Jump to main process */
	main();

	/* No actions to perform after this so wait forever */
//	while(1);
}



/********************************************************************/

