/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  DFlashSetProtection.c                               *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : DFlashSetProtection.c
*  Description      : This function sets the D-Flash protection to the 
*					            intended protection status 
*  Arguments        : PFLASH_SSD_CONFIG, UINT8
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 DFlashSetProtection(PFLASH_SSD_CONFIG PSSDConfig, UINT8 protectStatus)
{
	UINT8  registerValue;    /* store data read from flash register */
  UINT32 returnCode;       /* return code variable */
	UINT8  temp;
	
	/* set the default return as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* Check if size of DFlash = 0 */
	if(PSSDConfig->DFlashBlockSize == 0)
	{
		returnCode = FTFL_ERR_EFLASHONLY;
		goto EXIT;
	}
	else
	{
		/* Do nothing */
	}

	/* Read the value of FDPROT register */
	registerValue = REG_READ(PSSDConfig->ftflRegBase + FTFL_FDPROT_OFFSET);
	
	/* Check if normal mode */
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FDPROT_OFFSET, 0xFF);
	
	if(REG_READ(PSSDConfig->ftflRegBase + FTFL_FDPROT_OFFSET) != 0xFF)
	{
	/* if normal mode */
		temp = (UINT8)(protectStatus ^ registerValue);
		if(!(temp & protectStatus))
		{
			REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FDPROT_OFFSET, protectStatus);
		}
		else
		{
			returnCode = FTFL_ERR_CHANGEPROT;
		}
		goto EXIT;
	}
	/* if unprotected or special mode */
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FDPROT_OFFSET, protectStatus);
	
EXIT:
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */