/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  EERAMGetProtection.c                                *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"


/************************************************************************
*
*  Function Name    : EERAMGetProtection.c
*  Description      : This function retrieves current EERAM protection status. 
*  Arguments        : PFLASH_SSD_CONFIG, UINT8*
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 EERAMGetProtection(PFLASH_SSD_CONFIG PSSDConfig, UINT8* protectStatus)
{
	UINT32 returnCode;      /* return code variable */

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* Check if EERAM is set for EEE */
	if(REG_BIT_TEST(PSSDConfig->ftflRegBase + FTFL_FCNFG_OFFSET, FTFL_FCNFG_EEERDY))
	{
		/* EERAM is set for EEE */
		*protectStatus = REG_READ(PSSDConfig->ftflRegBase + FTFL_FEPROT_OFFSET);
	}
	else
	{
		returnCode = FTFL_ERR_NOEEE;
		goto EXIT;
	}

EXIT:
	  /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */