/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  EERAMGetProtection.c                                *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : EERAMGetProtection.c
*  Description      : This function retrieves current EERAM protection status. 
*  Arguments        : PFLASH_SSD_CONFIG, UINT8
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 EERAMSetProtection(PFLASH_SSD_CONFIG PSSDConfig, UINT8 protectStatus)
{
	UINT8  registerValue;    /* store data read from flash register */
  UINT32 returnCode;       /* return code variable */
	UINT8  temp;
	
	/* set the default return as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* Check if EERAM is set for EEE */
	if(!(REG_BIT_TEST(PSSDConfig->ftflRegBase + FTFL_FCNFG_OFFSET, FTFL_FCNFG_EEERDY)))
	{
		/* EERAM is not set for EEE */
		returnCode = FTFL_ERR_NOEEE;
		goto EXIT;
	}
	else
	{
		/* Do nothing */
	}
	
	/* Read the value of FDPROT register */
	registerValue = REG_READ(PSSDConfig->ftflRegBase + FTFL_FEPROT_OFFSET);
	
	/* Check if normal mode */
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FEPROT_OFFSET, 0xFF);
	
	if(REG_READ(PSSDConfig->ftflRegBase + FTFL_FEPROT_OFFSET) != 0xFF)
	{
	/* if normal mode */
		temp = (UINT8)(protectStatus ^ registerValue);
		if(!(temp & protectStatus))
		{
			REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FEPROT_OFFSET, protectStatus);
		}
		else
		{
			returnCode = FTFL_ERR_CHANGEPROT;
		}
		goto EXIT;
	}
	/* if unprotected or special mode */
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FEPROT_OFFSET, protectStatus);
	
EXIT:
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */