/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashEraseAllBlock.c                                *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

//#pragma location = ".textrw_init"
/************************************************************************
*
*  Function Name    : FlashEraseAllBlock.c
*  Description      : The Erase All Blocks operation will erase all Flash
					            memory, initialize the EERAM, verify all memory
					            contents, then release MCU security.
*  Arguments        : PFLASH_SSD_CONFIG, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashEraseAllBlock (PFLASH_SSD_CONFIG PSSDConfig, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
    // VUINT8 registerValue;   /* store data read from flash register */
    UINT8 commandArray[1];     /* command sequence array */
    UINT32 returnCode;         /* return code variable */

    /* set the default return code as FTFL_OK */
    returnCode = FTFL_OK;

    /* preparing passing parameter to erase all flash blocks */
    /* 1st element for the FCCOB register */
    commandArray[0] = FTFL_ERASE_ALL_BLOCK;

    /* calling flash command sequence function to execute the command */
    returnCode = FlashCommandSequence(PSSDConfig, 0, commandArray);

    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}


/* end of file */