/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashEraseSuspend.c                                 *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashEraseSuspend.c
*  Description      : This function is used to suspend a current operation
*					            of flash erase sector command.
*  Arguments        : PFLASH_SSD_CONFIG
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashEraseSuspend(PFLASH_SSD_CONFIG PSSDConfig)
{
  UINT32 returnCode;      /* return code variable */

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
  
	/* check CCIF bit of the flash status register */
  if(FALSE == (REG_BIT_TEST(PSSDConfig->ftflRegBase + FTFL_FSTAT_OFFSET, FTFL_FSTAT_CCIF)))
  {
  	/* If the command write sequence in progressing, */
  	/* Set ERSSUSP bit in FCNFG register */
  	REG_BIT_SET(PSSDConfig->ftflRegBase + FTFL_FCNFG_OFFSET, FTFL_FCNFG_ERSSUSP);
  }
	else
	{
		/* Do nothing */
	}
	
	/* Enter Debug state if enabled */
  if (TRUE == (PSSDConfig->DebugEnable))
  {
       asm
       (
        " BKPT #0 \n "           /* enter Debug state */
       );
  }

  return(returnCode);
}
/* end of file */