/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashGetInterruptEnable.c                           *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/*********************************************************************
*
*  Function Name    : FlashGetInterruptEnable
*  Description      : This function will read the FCNFG register and 
*					            return the interrupt enable states for Flash 
*					            interrupt types.
*  Arguments        : PFLASH_SSD_CONFIG, UINT8*
*  Return Value     : UINT32
*
**********************************************************************/

UINT32 FlashGetInterruptEnable(PFLASH_SSD_CONFIG PSSDConfig, UINT8* interruptState)
{
    /* store data read from flash register */
    UINT8  registerValue;

    /* Initial interrupt state value */
    *interruptState = 0;

    /*Get flash configration register value */
    registerValue = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCNFG_OFFSET);

    /* Add command complete interrupt enable bit value */
    /* CCIE bit of the FCNFG register */
    *interruptState |= (registerValue & (FTFL_FCNFG_CCIE | FTFL_FCNFG_RDCOLLIE));

    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(FTFL_OK);
}
/* end of file */

