/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashProgramByte.c                                  *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashProgramByte.c
*  Description      : Program data into Flash
*  Arguments        : PFLASH_SSD_CONFIG, UINT32, UINT32, UINT32,
*                     pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashProgramByte(PFLASH_SSD_CONFIG PSSDConfig, \
												UINT32 destination, \
												UINT32 size, \
												UINT32 source, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
  UINT8 pCommandArray[5]; /* command sequence array */
  UINT32 returnCode;      /* return code variable */
  UINT32 endAddress;      /* storing end address */

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* calculating Flash end address */
  endAddress = destination + size;
	
	/* check for valid range of the target addresses */
	if((destination < PSSDConfig->PFlashBlockBase) || \
		(endAddress > (PSSDConfig->PFlashBlockBase + PSSDConfig->PFlashBlockSize)))
	{
		if((destination < PSSDConfig->DFlashBlockBase) || \
			(endAddress > (PSSDConfig->DFlashBlockBase + PSSDConfig->DFlashBlockSize)))
		{
			/* return an error code FTFL_ERR_RANGE */
			returnCode = FTFL_ERR_RANGE;
			goto EXIT;
		}
		else
		{
			/* Convert System memory address to FTFL internal memory address */
			destination = destination - PSSDConfig->DFlashBlockBase + 0x800000;
		}
	}
	else
  {
	/* Convert System memory address to FTFL internal memory address */
      destination -= PSSDConfig->PFlashBlockBase;
  }

  /* check for error return code */
  if(FTFL_OK == returnCode)
  {
      while(size > 0)
      {
  			/* preparing passing parameter to program the flash block */
  			pCommandArray[0] = FTFL_PROGRAM_BYTE;
  			pCommandArray[1] = (UINT8)(destination >> 16);
  			pCommandArray[2] = (UINT8)((destination >> 8) & 0xFF);
  			pCommandArray[3] = (UINT8)(destination & 0xFF);
  			pCommandArray[4] = READ8(source);
              		
  			/* calling flash command sequence function to execute the command */
  			returnCode = FlashCommandSequence(PSSDConfig, 4, pCommandArray);

        /* checking for the success of command execution */
        if(FTFL_OK != returnCode)
        {
            break;
        }
        else
        {
            /* update destination address for next iteration */
            destination += 1;
            /* update size for next iteration */
            size -= 1;
		        /* increment the source adress by 1 */
            source += 1;
        }
      }
  }

EXIT:
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */


