/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashProgramCheck.c                                 *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashProgramCheck.c
*  Description      : The Program Check command tests a previously 
*					            programmed P-Flash or D-Flash longword to see 
*					            if it reads correctly at the specifed margin level.
*  Arguments        : PFLASH_SSD_CONFIG, UINT32,UINT32, UINT8*, UINT32*,
*                     UINT8*, UINT8, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashProgramCheck(PFLASH_SSD_CONFIG PSSDConfig, \
				UINT32  destination, \
					UINT32  size, \
				UINT8*  pExpectedData, \
				UINT32* pFailAddr, \
				UINT8*  pFailData, \
				UINT8   marginLevel, \
		pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
  UINT8 pCommandArray[12]; /* command sequence array */
  UINT32 returnCode;      /* return code variable */
  
  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
  
 	while (size)
	{


          if(destination >= (PSSDConfig->PFlashBlockBase + PSSDConfig->PFlashBlockSize)){
	/* Convert System memory address to FTFL internal memory address */
	destination = destination - PSSDConfig->DFlashBlockBase + 0x800000;

        }


	  
      /* check for error return code */
      if(FTFL_OK == returnCode)
      {
    		/* preparing passing parameter to program check the flash block */
    		pCommandArray[0] = FTFL_PROGRAM_CHECK;
    		pCommandArray[1] = (UINT8)(destination >> 16);
    		pCommandArray[2] = (UINT8)((destination >> 8) & 0xFF);
    		pCommandArray[3] = (UINT8)(destination & 0xFF);
    		pCommandArray[4] = marginLevel;		
    		pCommandArray[11] = *pExpectedData;
    		pCommandArray[10] = *(pExpectedData+1);
    		pCommandArray[9] = *(pExpectedData+2);
    		pCommandArray[8] = *(pExpectedData+3);
    		
    		/* calling flash command sequence function to execute the command */
    		returnCode = FlashCommandSequence(PSSDConfig, 11, pCommandArray);
    		
    		/* checking for the success of command execution */
    		if(FTFL_OK != returnCode)
    		{
                     goto EXIT;
    		}
    	
        }
            
        size-=FTFL_LONGWORD_SIZE;
        pExpectedData+=FTFL_LONGWORD_SIZE;
        destination+= FTFL_LONGWORD_SIZE;
    }

EXIT:

    return(returnCode);
}

/* end of file */


