/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashProgramLongword.c                              *
* DATE          :  August 25,2010                                      *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashProgramLongword.c
*  Description      : Program data into Flash
*  Arguments        : PFLASH_SSD_CONFIG, UINT32, UINT32, UINT32,
*                     pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashProgramLongword(PFLASH_SSD_CONFIG PSSDConfig, \
												UINT32 destination, \
												UINT32 size, \
												UINT32 source, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
  UINT8 pCommandArray[8]; /* command sequence array */
  UINT32 returnCode;      /* return code variable */
  UINT32 endAddress;      /* storing end address */
  UINT32 data;

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* calculating Flash end address */
  endAddress = destination + size;
	
	
	/* check for valid range of the target addresses */
	if((destination < PSSDConfig->PFlashBlockBase) || \
		(endAddress > (PSSDConfig->PFlashBlockBase + PSSDConfig->PFlashBlockSize)))
	{
				/* Convert System memory address to FTFL internal memory address */
	  destination = destination - PSSDConfig->DFlashBlockBase + 0x800000;
		
	}
	else
  {
      /* Convert System memory address to FTFL internal memory address */
      destination -= PSSDConfig->PFlashBlockBase;
  }
	
  /* check for error return code */
  if(FTFL_OK == returnCode)
  {
    while(size > 0)
    {
			/* preparing passing parameter to program the flash block */
			pCommandArray[0] = FTFL_PROGRAM_LONGWORD;
			pCommandArray[1] = (UINT8)(destination >> 16);
			pCommandArray[2] = (UINT8)((destination >> 8) & 0xFF);
			pCommandArray[3] = (UINT8)(destination & 0xFF);
      
      data = READ32(source);
      
      pCommandArray[4] = bits_31_24(data); 
      pCommandArray[5] = bits_23_16(data); 
      pCommandArray[6] = bits_15_8(data);  
      pCommandArray[7] = bits_7_0(data);   
   		
			/* calling flash command sequence function to execute the command */
			returnCode = FlashCommandSequence(PSSDConfig, 7, pCommandArray);

      /* checking for the success of command execution */
      if(FTFL_OK != returnCode)
      {
          break;
      }
      else
      {
        /* update destination address for next iteration */
        destination += FTFL_LONGWORD_SIZE;
        /* update size for next iteration */
        size -= FTFL_LONGWORD_SIZE;
				/* increment the source adress by 1 */
        source += FTFL_LONGWORD_SIZE;
      }
    }
  }

EXIT:

    return(returnCode);
}
/* end of file */


