/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashProgramOnce.c                                  *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashProgramOnce.c
*  Description      : Program data into a dedicated 64 bytes region in
*                     the P-Flash IFR which stores critical information
*                     for the user
*  Arguments        : PFLASH_SSD_CONFIG, UINT8*, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashProgramOnce(PFLASH_SSD_CONFIG PSSDConfig, \
												UINT8* pDataArray, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
    UINT8  counter;          /* outer loop counter */
    UINT8  pCommandArray[8]; /* command sequence array */
    UINT32 returnCode;       /* return code variable */

    /* set the default return code as FTFL_OK */
    returnCode = FTFL_OK;
    /* set the counter value for 0 */
    counter = 0;

    /* preparing passing parameter for program once command */
    /* 1st element for the FCCOB register */
	  pCommandArray[0] = FTFL_PROGRAM_ONCE;

    while (counter <= 0x0F)
    {
    		/* preparing the rest of the parameters to be passed for the FCCOB registers */
    		pCommandArray[1] = counter;
    		pCommandArray[2] = 0x00;
    		pCommandArray[3] = 0x00;
    		pCommandArray[4] = pDataArray[counter*4];
    		pCommandArray[5] = pDataArray[counter*4 + 1];
        pCommandArray[6] = pDataArray[counter*4 + 2];
    		pCommandArray[7] = pDataArray[counter*4 + 3];

        /* calling flash command sequence API to execute the command */
        returnCode = FlashCommandSequence(PSSDConfig, 7, pCommandArray);

        /* checking for the success of command execution */
        if(FTFL_OK != returnCode)
        {
            break;
        }
        else
        {
            /* do nothing */
        }
        /* decrement the counter value */
        counter++;
    }

    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */