/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashReadOnce.c                                     *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashReadOnce.c
*  Description      : This function is used to read access to a reserved 
*					            64 byte field located in the P-Flash IFR.
*  Arguments        : PFLASH_SSD_CONFIG, UINT8*, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashReadOnce(PFLASH_SSD_CONFIG PSSDConfig, \
												UINT8* pDataArray, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
    UINT8  counter;          /* outer loop counter */
    UINT8  pCommandArray[2]; /* command sequence array */
    UINT32 returnCode;       /* return code variable */

    /* initialize counter and returnCode */
    counter = 0;

    /* set the default return code as FTFL_OK */
    returnCode = FTFL_OK;

    /* preparing passing parameter for read once command */
    /* 1st element for the FCCOB register */
    pCommandArray[0] = FTFL_READ_ONCE;

    while (counter <= 0x0F)
    {
        /* 2nd element for the FCCOB register */
        pCommandArray[1] = counter;

        /* calling flash command sequence API to execute the command */
        returnCode = FlashCommandSequence(PSSDConfig, 1, pCommandArray);
		
		    /* checking for the success of command execution */
        if(FTFL_OK != returnCode)
        {
            break;
        }
        else
        {
            /* do nothing */
        }
		
        /* Read the data from the FCCOB registers into the pDataArray */
        pDataArray[counter*4] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB4_OFFSET);
    		pDataArray[counter*4 + 1] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB5_OFFSET);
    		pDataArray[counter*4 + 2] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB6_OFFSET);
    		pDataArray[counter*4 + 3] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB7_OFFSET);

        counter++;
    }

    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */