/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashReadResource.c                                 *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/************************************************************************
*
*  Function Name    : FlashReadResource.c
*  Description      : This function is provided for the user to read data
*					            from P-Flash IFR and D-Flash IFR space.
*  Arguments        : PFLASH_SSD_CONFIG, UINT32, UINT8*, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashReadResource(PFLASH_SSD_CONFIG PSSDConfig, \
												UINT32 destination, \
												UINT8* pDataArray, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
  UINT8  pCommandArray[9]; /* command sequence array */
  UINT32 returnCode;       /* return code variable */

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* check if the destination is longword aligned or not */
  if (0 != destination % FTFL_LONGWORD_SIZE)
  {
      /* return an error code FTFL_ERR_ADDR */
      returnCode = FTFL_ERR_ADDR;
      goto EXIT;
  }
	
	/* check for valid range of the target addresses */	
	if((destination < PFLASH_IFR_START_ADDRESS) || \
		(destination >= PFLASH_IFR_END_ADDRESS))
	{
		if((destination < DFLASH_IFR_START_ADDRESS) || \
			(destination >= DFLASH_IFR_END_ADDRESS))
		{
			/* return an error code FTFL_ERR_RANGE */
			returnCode = FTFL_ERR_RANGE;
			goto EXIT;
		}
		else
		{
			/* Convert System memory address to FTFL internal memory address */
			//destination = destination - PSSDConfig->DFlashBlockBase + 0x800000;
		}
	}
	else
	{
		/* Convert System memory address to FTFL internal memory address */
        //destination -= PSSDConfig->PFlashBlockBase;
	}

  /* preparing passing parameter for read once command */
  /* 1st element for the FCCOB register */
  pCommandArray[0] = FTFL_READ_RESOURCE;
	pCommandArray[1] = (UINT8)(destination >> 16);
	pCommandArray[2] = (UINT8)((destination >> 8) & 0xFF);
	pCommandArray[3] = (UINT8)(destination & 0xFF);
	pCommandArray[8] = 0x00;
	
	/* calling flash command sequence API to execute the command */
	returnCode = FlashCommandSequence(PSSDConfig, 8, pCommandArray);
	
	/* checking for the success of command execution */
	if(FTFL_OK != returnCode)
	{
		goto EXIT;
	}
	else
	{
		/* do nothing */
	}
	
	/* Read the data from the FCCOB registers into the pDataArray */
	pDataArray[0] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB4_OFFSET);
	pDataArray[1] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB5_OFFSET);
	pDataArray[2] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB6_OFFSET);
	pDataArray[3] = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCCOB7_OFFSET);

EXIT:
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */