/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashSetInterruptEnable.c                           *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"

/*********************************************************************
*
*  Function Name    : FlashSetInterruptEnable
*  Description      : This function sets the Flash interrupt enable 
*					            bits in the Flash module configuration register.
*					            Other bits in the FCNFG register wont be affected.
*  Arguments        : PFLASH_SSD_CONFIG, UINT8
*  Return Value     : UINT32
*
**********************************************************************/

UINT32 FlashSetInterruptEnable(PFLASH_SSD_CONFIG PSSDConfig, UINT8 interruptState)
{
    /* store data read from flash register */
    UINT8  registerValue;

    /* Mask off unused bits in interruptState */
    interruptState &= (FTFL_FCNFG_CCIE | FTFL_FCNFG_RDCOLLIE);

    /*Get flash configration register value */
    registerValue = REG_READ(PSSDConfig->ftflRegBase + FTFL_FCNFG_OFFSET);

    /* Set register value with desired interrupt state */
    registerValue &= ~(FTFL_FCNFG_CCIE | FTFL_FCNFG_RDCOLLIE);
    registerValue |= interruptState;
    REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FCNFG_OFFSET, registerValue);

    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(FTFL_OK);
}
/* end of file */






