/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashVerifyAllBlock.c                               *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"


/************************************************************************
*
*  Function Name    : FlashVerifyAllBlock.c
*  Description      : This function will check to see if the P-Flash 
*					            and D-Flash blocks as well as EERAM, E-Flash records
*					            and D-Flash IFR have been erased to the specifed read
*					            margin level, if applicable, and will release security
*					            if the readout passes.
*  Arguments        : PFLASH_SSD_CONFIG,UINT8, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashVerifyAllBlock(PFLASH_SSD_CONFIG PSSDConfig, \
												UINT8 marginLevel, \
												pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
  UINT8 pCommandArray[2]; /* command sequence array */
  UINT32 returnCode;      /* return code variable */

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
	
	/* preparing passing parameter to verify all block command*/
  pCommandArray[0] = FTFL_VERIFY_ALL_BLOCK;
	pCommandArray[1] = marginLevel;

  /* calling flash command sequence function to execute the command */
  returnCode = FlashCommandSequence(PSSDConfig, 1, pCommandArray);

 /* Giving Control to BDM if enabled */
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */