/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  FlashVerifySection.c                                *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"


/************************************************************************
*
*  Function Name    : FlashVerifySection.c
*  Description      : This function will check to see if a section of
*					            P-Flash or D-Flash memory is erased to the specifed 
*					            read margin level.
*  Arguments        : PFLASH_SSD_CONFIG,UINT32,UINT16,UINT8, pFLASHCOMMANDSEQUENCE
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 FlashVerifySection(PFLASH_SSD_CONFIG PSSDConfig, \
                            UINT32 destination, \
                            UINT16 Number, \
                            UINT8 marginLevel, \
                            pFLASHCOMMANDSEQUENCE FlashCommandSequence)
{
  UINT8 pCommandArray[7]; /* command sequence array */
  UINT32 returnCode;      /* return code variable */
  UINT32 endAddress;      /* storing end address. This is the starting point of the next block */	
  UINT32 AlignedChecking;

  /* set the default return code as FTFL_OK */
  returnCode = FTFL_OK;
	
  /*Different derivative will cause different alignment checking*/
  #if(FTFL_LL_128K_32K_2K==FLASH_DERIVATIVE)
   
       AlignedChecking= FTFL_LONGWORD_SIZE;
   
  #else
   
       AlignedChecking= FTFL_PHRASE_SIZE;
    
  #endif
	
	 /* calculating Flash end address */
  endAddress = destination + Number * AlignedChecking;
	/* check if the destination is phrase aligned or not */
  if (0 != (destination % AlignedChecking))
    {
      /* return an error code FTFL_ERR_ADDR */
      returnCode = FTFL_ERR_ADDR;
      goto EXIT;
    }
	
	/* check for valid range of the target addresses */
	if((destination < PSSDConfig->PFlashBlockBase) || \
		(endAddress > (PSSDConfig->PFlashBlockBase + PSSDConfig->PFlashBlockSize)))
	{
		if((destination < PSSDConfig->DFlashBlockBase) || \
			(endAddress > (PSSDConfig->DFlashBlockBase + PSSDConfig->DFlashBlockSize)))
		{
			/* return an error code FTFL_ERR_RANGE */
			returnCode = FTFL_ERR_RANGE;
			goto EXIT;
		}
		else
		{
			/* Convert System memory address to FTFL internal memory address */
			destination = destination - PSSDConfig->DFlashBlockBase + 0x800000;
		}
	}
	else
  {
    /* Convert System memory address to FTFL internal memory address */
    destination -= PSSDConfig->PFlashBlockBase;
  }
		
	/* preparing passing parameter to verify section command */
  pCommandArray[0] = FTFL_VERIFY_SECTION;
	pCommandArray[1] = (UINT8)(destination >> 16);
	pCommandArray[2] = (UINT8)((destination >> 8) & 0xFF);
	pCommandArray[3] = (UINT8)(destination & 0xFF);
	pCommandArray[4] = (UINT8)(Number >> 8);
	pCommandArray[5] = (UINT8)(Number & 0xFF);
	pCommandArray[6] = marginLevel;

  /* calling flash command sequence function to execute the command */
  returnCode = FlashCommandSequence(PSSDConfig, 6, pCommandArray);

EXIT:
 
    return(returnCode);
}
/* end of file */