/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  PFlashGetProtection.c                               *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"


/************************************************************************
*
*  Function Name    : PFlashGetProtection.c
*  Description      : This function retrieves current P-Flash protection status. 
*  Arguments        : PFLASH_SSD_CONFIG, UINT32*
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 PFlashGetProtection(PFLASH_SSD_CONFIG PSSDConfig, UINT32* protectStatus)
{
  UINT32 registerValue0;
  UINT32 registerValue1;
  UINT32 registerValue2;
  UINT32 registerValue3;
  
  registerValue0=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT0_OFFSET));
  registerValue1=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT1_OFFSET));
  registerValue2=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT2_OFFSET));
  registerValue3=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT3_OFFSET));
  
  *protectStatus = ((registerValue0 << 24 ) | \
                   (registerValue1 << 16 ) | \
                   (registerValue2 << 8 ) | \
                    registerValue3); 
                   
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

  return(FTFL_OK);
}
/* end of file */