/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For FTFL           		         *
*                                                                      *
* FILE NAME     :  PFlashSetProtection.c                               *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"


/************************************************************************
*
*  Function Name    : PFlashSetProtection.c
*  Description      : This function sets the P-Flash protection to the 
*					            intended protection status 
*  Arguments        : PFLASH_SSD_CONFIG, UINT32
*  Return Value     : UINT32
*
*************************************************************************/

UINT32 PFlashSetProtection(PFLASH_SSD_CONFIG PSSDConfig, \
                                                UINT32  protectStatus)
{
	UINT32 registerValue;    /* store data read from flash register */
  UINT32 returnCode;       /* return code variable */
	UINT32 temp;
	UINT32 registerValue0;
  UINT32 registerValue1;
  UINT32 registerValue2;
  UINT32 registerValue3;
  
  /* set the default return as FTFL_OK */
  returnCode = FTFL_OK;
  
  /* Read the value of FDPROT registers */
  registerValue0=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT0_OFFSET));
  registerValue1=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT1_OFFSET));
  registerValue2=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT2_OFFSET));
  registerValue3=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT3_OFFSET));
	
	registerValue = ((registerValue0 << 24 ) | \
                   (registerValue1 << 16 ) | \
                   (registerValue2 << 8 ) | \
                    registerValue3); 
	
	/* Check if normal mode */
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT0_OFFSET, 0xFF);
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT1_OFFSET, 0xFF);
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT2_OFFSET, 0xFF);
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT3_OFFSET, 0xFF);
  
  /* Read the value of FDPROT registers */
  registerValue0=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT0_OFFSET));
  registerValue1=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT1_OFFSET));
  registerValue2=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT2_OFFSET));
  registerValue3=(UINT32)(REG_READ(PSSDConfig->ftflRegBase + FTFL_FPROT3_OFFSET));
	
	if((registerValue0 != 0xFF) | (registerValue1 != 0xFF) | \
	   (registerValue2 != 0xFF) | (registerValue3 != 0xFF))
	{
	/* if normal mode */
		temp = protectStatus ^ registerValue;
		if(!(temp & protectStatus))
		{
			REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT0_OFFSET, (UINT8)(protectStatus >> 24));
			REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT1_OFFSET, (UINT8)(protectStatus >> 16));
			REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT2_OFFSET, (UINT8)(protectStatus >> 8));
			REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT3_OFFSET, (UINT8)protectStatus);			
		}
		else
		{
			returnCode = FTFL_ERR_CHANGEPROT;
		}
		goto EXIT;
	}
	/* if unprotected or special mode */
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT0_OFFSET, (UINT8)(protectStatus >> 24));
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT1_OFFSET, (UINT8)(protectStatus >> 16));
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT2_OFFSET, (UINT8)(protectStatus >> 8));
	REG_WRITE(PSSDConfig->ftflRegBase + FTFL_FPROT3_OFFSET, (UINT8)protectStatus);
	
EXIT:
    /* Enter Debug state if enabled */
    if (TRUE == (PSSDConfig->DebugEnable))
    {
         asm
         (
          " BKPT #0 \n "           /* enter Debug state */
         );
    }

    return(returnCode);
}
/* end of file */