/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

************************************************************************
*                                                                      *
*        Standard Software Flash Driver For C90TFS           		       *
*                                                                      *
* FILE NAME     :  SSD_C90TFS_Internal.h                               *
* DATE          :  June 11,2010                                        *
*                                                                      *
* AUTHOR        :  FPT Team                                            *
* E-mail        :  b28216@freescale.com                                *
*                                                                      *
************************************************************************/

/************************** CHANGES ***********************************
0.0.1    	06.09.2010    	FPT Team            		Initial Version
0.1.0    	06.11.2010    	FPT Team            		Finalize to 0.1.0 
***********************************************************************/

#ifndef _SSD_C90TFS_INTERNAL_H_
#define _SSD_C90TFS_INTERNAL_H_

/*--------------- C90TFS Flash Module Memory Offset Map -----------------*/				 
#if(FTFL_LL_128K_32K_2K == FLASH_DERIVATIVE)    //LLTV2 - big endian
	/* Flash Status Register (FSTAT)*/
	#define C90TFS_FSTAT_OFFSET              0x00000003
	#define C90TFS_FSTAT_CCIF              	 0x80
	#define C90TFS_FSTAT_RDCOLERR          	 0x40
	#define C90TFS_FSTAT_ACCERR				       0x20
	#define C90TFS_FSTAT_FPVIOL			      	 0x10
	#define C90TFS_FSTAT_MGSTAT0		       	 0x01

	/* Flash configuration register (FCNFG)*/
	#define C90TFS_FCNFG_OFFSET              0x00000002
	#define C90TFS_FCNFG_CCIE                0x80
	#define C90TFS_FCNFG_RDCOLLIE			       0x40
	#define C90TFS_FCNFG_ERSAREQ             0x20
	#define C90TFS_FCNFG_ERSSUSP             0x10
	#define C90TFS_FCNFG_RAMRDY              0x02
	#define C90TFS_FCNFG_EEERDY              0x01

	/* Flash security register (FSEC) */
	#define C90TFS_FSEC_OFFSET               0x00000001
	#define C90TFS_FSEC_KEYEN                0xC0
	#define C90TFS_FSEC_FSLACC				       0x0C
	#define C90TFS_FSEC_SEC                  0x03

	/* Flash Option Register (FOPT) */
	#define C90TFS_FOPT_OFFSET                0x00000000

	/* Flash common command object registers (FCCOB0-B) */
	#define C90TFS_FCCOB0_OFFSET             0x00000004
	#define C90TFS_FCCOB1_OFFSET             0x00000005
	#define C90TFS_FCCOB2_OFFSET             0x00000006
	#define C90TFS_FCCOB3_OFFSET             0x00000007
	#define C90TFS_FCCOB4_OFFSET             0x00000008
	#define C90TFS_FCCOB5_OFFSET             0x00000009
	#define C90TFS_FCCOB6_OFFSET             0x0000000A
	#define C90TFS_FCCOB7_OFFSET             0x0000000B
	#define C90TFS_FCCOB8_OFFSET             0x0000000C
	#define C90TFS_FCCOB9_OFFSET             0x0000000D
	#define C90TFS_FCCOBA_OFFSET             0x0000000E
	#define C90TFS_FCCOBB_OFFSET             0x0000000F

	/* P-Flash protection registers (FPROT0-3) */
	#define C90TFS_FPROT0_OFFSET             0x00000010
	#define C90TFS_FPROT1_OFFSET             0x00000011
	#define C90TFS_FPROT2_OFFSET             0x00000012
	#define C90TFS_FPROT3_OFFSET             0x00000013

	/* D-Flash protection registers (FDPROT) */
	#define C90TFS_FDPROT_OFFSET             0x00000014

	/* EERAM Protection Register (FEPROT)  */
	#define C90TFS_FEPROT_OFFSET             0x00000015
#else	//P2 products - little endian
	/* Flash Status Register (FSTAT)*/
	#define C90TFS_FSTAT_OFFSET              0x00000000
	#define C90TFS_FSTAT_CCIF              	 0x80
	#define C90TFS_FSTAT_RDCOLERR          	 0x40
	#define C90TFS_FSTAT_ACCERR				       0x20
	#define C90TFS_FSTAT_FPVIOL			      	 0x10
	#define C90TFS_FSTAT_MGSTAT0		       	 0x01

	/* Flash configuration register (FCNFG)*/
	#define C90TFS_FCNFG_OFFSET              0x00000001
	#define C90TFS_FCNFG_CCIE                0x80
	#define C90TFS_FCNFG_RDCOLLIE			       0x40
	#define C90TFS_FCNFG_ERSAREQ             0x20
	#define C90TFS_FCNFG_ERSSUSP             0x10
	#define C90TFS_FCNFG_RAMRDY              0x02
	#define C90TFS_FCNFG_EEERDY              0x01

	/* Flash security register (FSEC) */
	#define C90TFS_FSEC_OFFSET               0x00000002
	#define C90TFS_FSEC_KEYEN                0xC0
	#define C90TFS_FSEC_FSLACC				       0x0C
	#define C90TFS_FSEC_SEC                  0x03

	/* Flash Option Register (FOPT) */
	#define C90TFS_FOPT_OFFSET                0x00000003

	/* Flash common command object registers (FCCOB0-B) */
	#define C90TFS_FCCOB0_OFFSET             0x00000007
	#define C90TFS_FCCOB1_OFFSET             0x00000006
	#define C90TFS_FCCOB2_OFFSET             0x00000005
	#define C90TFS_FCCOB3_OFFSET             0x00000004
	#define C90TFS_FCCOB4_OFFSET             0x0000000B
	#define C90TFS_FCCOB5_OFFSET             0x0000000A
	#define C90TFS_FCCOB6_OFFSET             0x00000009
	#define C90TFS_FCCOB7_OFFSET             0x00000008
	#define C90TFS_FCCOB8_OFFSET             0x0000000F
	#define C90TFS_FCCOB9_OFFSET             0x0000000E
	#define C90TFS_FCCOBA_OFFSET             0x0000000D
	#define C90TFS_FCCOBB_OFFSET             0x0000000C

	/* P-Flash protection registers (FPROT0-3) */
	#define C90TFS_FPROT0_OFFSET             0x00000013
	#define C90TFS_FPROT1_OFFSET             0x00000012
	#define C90TFS_FPROT2_OFFSET             0x00000011
	#define C90TFS_FPROT3_OFFSET             0x00000010

	/* D-Flash protection registers (FDPROT) */
	#define C90TFS_FDPROT_OFFSET             0x00000017

	/* EERAM Protection Register (FEPROT)  */
	#define C90TFS_FEPROT_OFFSET             0x00000016
#endif

/*------------- Flash hardware algorithm operation commands -------------*/
#define C90TFS_VERIFY_BLOCK		           0x00
#define C90TFS_VERIFY_SECTION  		       0x01
#define C90TFS_PROGRAM_CHECK		         0x02
#define C90TFS_READ_RESOURCE             0x03
#define C90TFS_PROGRAM_BYTE              0x04
#define C90TFS_PROGRAM_WORD              0x05
#define C90TFS_PROGRAM_LONGWORD          0x06
#define C90TFS_ERASE_BLOCK	             0x08
#define C90TFS_ERASE_SECTOR		           0x09
#define C90TFS_PROGRAM_SECTION           0x0B
#define C90TFS_VERIFY_ALL_BLOCK          0x40
#define C90TFS_READ_ONCE                 0x41
#define C90TFS_PROGRAM_ONCE    			     0x43
#define C90TFS_ERASE_ALL_BLOCK           0x44
#define C90TFS_SECURITY_BY_PASS          0x45
#define C90TFS_PROGRAM_PARTITION		     0x80
#define C90TFS_SET_EERAM    			       0x81

/* Flash margin read settings */
#define C90TFS_MARGIN_NORMAL             0x0000
#define C90TFS_USER_MARGIN_LEVEL1        0x0001
#define C90TFS_USER_MARGIN_LEVEL0        0x0002
#define C90TFS_FACTORY_MARGIN_LEVEL0     0x0003
#define C90TFS_FACTORY_MARGIN_LEVEL1     0x0004

#endif  /* _SSD_C90TFS_INTERNAL_H_ */