#include "USB.h"
#include "USB_Desc.h"
#include "usb_icp.h"


extern UINT8 USB_InterfaceReq_Handler();

/* Arrays and global buffers */
#pragma data_alignment=512
tBDT tBDTtable[8];

UINT8 gu8EP0_OUT_ODD_Buffer[EP0_SIZE];
UINT8 gu8EP0_OUT_EVEN_Buffer[EP0_SIZE];
UINT8 gu8EP0_IN_ODD_Buffer[EP0_SIZE];
UINT8 gu8EP0_IN_EVEN_Buffer[EP0_SIZE];
UINT8 gu8EP1_OUT_ODD_Buffer[EP1_SIZE];
UINT8 gu8EP1_OUT_EVEN_Buffer[EP1_SIZE];
UINT8 gu8EP1_IN_ODD_Buffer[EP1_SIZE];
UINT8 gu8EP1_IN_EVEN_Buffer[EP1_SIZE];


UINT8 *BufferPointer[]=
{
    gu8EP0_OUT_ODD_Buffer,
    gu8EP0_OUT_EVEN_Buffer,
    gu8EP0_IN_ODD_Buffer,
    gu8EP0_IN_EVEN_Buffer,
    gu8EP1_OUT_ODD_Buffer,
    gu8EP1_OUT_EVEN_Buffer,
    gu8EP1_IN_ODD_Buffer,
    gu8EP1_IN_EVEN_Buffer,
 
};

const UINT8 cEP_Size[]=
{
    EP0_SIZE,    
    EP0_SIZE,    
    EP0_SIZE,    
    EP0_SIZE,    
    EP1_SIZE,    
    EP1_SIZE,    
    EP1_SIZE,    
    EP1_SIZE,    
 };

const UINT8* String_Table[4]=
{
    String_Descriptor0,
    String_Descriptor1,
    String_Descriptor2,
    String_Descriptor3
};




/* Global Variables for USB Handling */
UINT8 gu8USB_Flags;
UINT8 gu8USBClearFlags;
UINT8 *pu8IN_DataPointer;
UINT8 gu8IN_Counter;
UINT8 gu8USB_Toogle_flags;
UINT8 gu8USB_PingPong_flags;
UINT8 gu8USB_Configure;
UINT8 gu8USB_Interface;
UINT16 gu16Status;
UINT8 gu8Interface;
UINT8 gu8HALT_EP;
UINT8 gu8USB_State;
tUSB_Setup *Setup_Pkt;
 



/**********************************************************/
void USB_Init(void)@".usb_hid_icp"
{  
    /* Software Configuration */
    Setup_Pkt=(tUSB_Setup*)BufferPointer[bEP0OUT_ODD];
    gu8USB_State=uPOWER;

    /* MPU Configuration */
    MPU_CESR=0;                                     // MPU is disable. All accesses from all bus masters are allowed

    /* SIM Configuration */
    #ifdef USB_CLOCK_CLKIN
    FLAG_SET(SIM_SCGC5_PORTE_SHIFT,SIM_SCGC5); 
    PORTE_PCR26=(0|PORT_PCR_MUX(7));                // Enabling PTE26 as CLK input    
    #endif    
    
    #ifdef USB_CLOCK_PLL
    FLAG_SET(SIM_SOPT2_PLLFLLSEL_SHIFT,SIM_SOPT2);  // Select PLL output
    #endif
    
    #ifndef USB_CLOCK_CLKIN
    FLAG_SET(SIM_SOPT2_USBSRC_SHIFT,SIM_SOPT2);     // PLL/FLL selected as CLK source
    #endif

    SIM_CLKDIV2|=USB_FARCTIONAL_VALUE;              // USB Freq Divider
    SIM_SCGC4|=(SIM_SCGC4_USBOTG_MASK);             // USB Clock Gating

    
     /* USB Module Configuration */
    // Reset USB Module
    USB0_USBTRC0|=USB_USBTRC0_USBRESET_MASK;
    while(FLAG_CHK(USB_USBTRC0_USBRESET_SHIFT,USB0_USBTRC0)){};
    
    // Set BDT Base Register
    USB0_BDTPAGE1=(UINT8)((UINT32)tBDTtable>>8);
    USB0_BDTPAGE2=(UINT8)((UINT32)tBDTtable>>16);
    USB0_BDTPAGE3=(UINT8)((UINT32)tBDTtable>>24);

    // Clear USB Reset flag
    FLAG_SET(USB_ISTAT_USBRST_MASK,USB0_ISTAT);

    // Enable USB Reset Interrupt
//    FLAG_SET(USB_INTEN_USBRSTEN_SHIFT,USB0_INTEN);
    
    // Enable weak pull downs
    USB0_USBCTRL=0x40;
    USB0_USBTRC0|=0x40;
    USB0_CTL|=0x01;
    
    // Pull up enable
    FLAG_SET(USB_CONTROL_DPPULLUPNONOTG_SHIFT,USB0_CONTROL);    
}



/**********************************************************/
void EP_IN_Transfer(UINT8 u8EP,UINT8 *pu8DataPointer,UINT8 u8DataSize)@".usb_hid_icp"
{
    UINT8 *pu8EPBuffer;
    UINT8 u8EPSize; 
    UINT16 u16Length=0;    
    UINT8 u8EndPointFlag;    

    /* Adjust the buffer location */
    u8EndPointFlag=u8EP;
//    if(u8EP)
        u8EP=(UINT8)(u8EP<<2);              
        u8EP+=2;                     // u8Ep (buffer location) = (u8EP*2)+2, eg: EP0In <-> bufferPointer[2]

    /* Assign the proper EP buffer */
    pu8EPBuffer=BufferPointer[u8EP];
    
    /* Check if is a pending transfer */
    if(FLAG_CHK(fIN,gu8USBClearFlags))
    {
        pu8IN_DataPointer=pu8DataPointer;
        gu8IN_Counter=u8DataSize;       

        u16Length=(Setup_Pkt->bLength_h<<8)+Setup_Pkt->bLength_l ;
        if((u16Length < u8DataSize) && (u8EP==2))
        {
            gu8IN_Counter=Setup_Pkt->bLength_l;
        }
    }

    /* Check transfer Size */
//    if(gu8IN_Counter > cEP_Size[u8EP])
//    {
//        u8EPSize = cEP_Size[u8EP];
//        gu8IN_Counter-=cEP_Size[u8EP];
//        FLAG_CLR(fIN,gu8USBClearFlags);
//    }
//    else
//    {
        u8EPSize = gu8IN_Counter;
        gu8IN_Counter=0;
        FLAG_SET(fIN,gu8USBClearFlags);
//    }
    
    /* Copy User buffer to EP buffer */
    tBDTtable[u8EP].Cnt=(u8EPSize);
    
    while(u8EPSize--)
         *pu8EPBuffer++=*pu8IN_DataPointer++;
                                                             

    /* USB Flags Handling */
    if(FLAG_CHK(u8EndPointFlag,gu8USB_Toogle_flags))
    {
        tBDTtable[u8EP].Stat._byte= kUDATA0;
        FLAG_CLR(u8EndPointFlag,gu8USB_Toogle_flags);
    } 
    else
    {
        tBDTtable[u8EP].Stat._byte= kUDATA1;
        FLAG_SET(u8EndPointFlag,gu8USB_Toogle_flags);
    }

}


/**********************************************************/
UINT8 EP_OUT_Transfer(UINT8 u8EP,UINT8 *pu8DataPointer)@".usb_hid_icp"
{
    UINT8 *pu8EPBuffer;
    UINT8 u8EPSize; 
    

    /* Adjust the buffer location */
    u8EP = u8EP << 1;              // need this code for other Endpoints, (skip for EP0 only)
    u8EP++;                        // u8Ep (buffer location) = (u8EP*2)+1, eg: EP0Out <-> bufferPointer[1] 

    /* Assign the proper EP buffer */
    pu8EPBuffer=BufferPointer[u8EP];
    
    /* Copy User buffer to EP buffer */
    u8EPSize=tBDTtable[u8EP].Cnt;
    u8EP=u8EPSize;
    
    while(u8EPSize--)
         *pu8DataPointer++=*pu8EPBuffer++;
    return(u8EP);
}

/**********************************************************/
UINT16 USB_EP_OUT_SizeCheck(UINT8 u8EP)
{
    UINT8 u8EPSize; 
    /* Read Buffer Size */
    u8EPSize=SWAP16(tBDTtable[u8EP<<2].Cnt);
    return(u8EPSize&0x03FF);
}

/**********************************************************/
void USB_Set_Interface(void)
{
    /* EndPoint Register settings */
    USB0_ENDPT1= EP1_VALUE | USB_ENDPT_EPHSHK_MASK;                         
 

    /* EndPoint 1 BDT Settings*/
    tBDTtable[bEP1IN_ODD].Stat._byte= kMCU;
    tBDTtable[bEP1IN_ODD].Cnt = 0x00;
    tBDTtable[bEP1IN_ODD].Addr =(UINT32)gu8EP1_IN_ODD_Buffer;

         
}

/**********************************************************/
void USB_StdReq_Handler(void)
{

    if (Setup_Pkt->bRequest ==mSET_ADDRESS )
    {
             EP_IN_Transfer(EP0,0,0);
            gu8USB_State=uADDRESS;
    }
      
    else if  (Setup_Pkt->bRequest ==mGET_DESC)
    {
            if (Setup_Pkt->bValue_h == mDEVICE) 
            {
               EP_IN_Transfer(EP0,(UINT8*)Device_Descriptor,sizeof(Device_Descriptor));
            }
                    
            else if(Setup_Pkt->bValue_h == mCONFIGURATION)
            {
 //                   EP_IN_Transfer(EP0,(UINT8*)Configuration_Descriptor,sizeof(Configuration_Descriptor));
                   EP_IN_Transfer(EP0,(UINT8*)Configuration_Descriptor,34);
            }
        
            else if (Setup_Pkt->bValue_h == mSTRING)
            {
                    EP_IN_Transfer(EP0,(UINT8*)String_Table[Setup_Pkt->bValue_l],String_Table[Setup_Pkt->bValue_l][0]);
            }

            else
            {  
               
              USB_InterfaceReq_Handler();

            }
    }

    else if (Setup_Pkt->bRequest ==mSET_CONFIG)
    {
           gu8USB_Configure=Setup_Pkt->bValue_h+Setup_Pkt->bValue_l;
            if(Setup_Pkt->bValue_h+Setup_Pkt->bValue_l) 
            {
                USB_Set_Interface();                         

                gu8USB_State=uENUMERATED;

            }
           EP_IN_Transfer(EP0,0,0);
    }
    
    else if (Setup_Pkt->bRequest ==mGET_CONFIG)
    {
            EP_IN_Transfer(EP0,(UINT8*)&gu8USB_Configure,1);
    }

   else if (Setup_Pkt->bRequest == mSET_INTF)
    {
           gu8USB_Interface=Setup_Pkt->bValue_h+Setup_Pkt->bValue_l;
                EP_IN_Transfer(EP0,0,0);
    }
      
    else if (Setup_Pkt->bRequest ==mGET_INTF)
    {
            EP_IN_Transfer(EP0,(UINT8*)&gu8USB_Interface,1);
    }
    
    else if (Setup_Pkt->bRequest ==mSET_FEATURE)
    {
            gu16Status = 0x0001;
            EP_IN_Transfer(EP0,0,0);
    }

    else if (Setup_Pkt->bRequest ==mCLR_FEATURE)
    {
            gu16Status = 0x0000;
            EP_IN_Transfer(EP0,0,0);
    }
    
    
    else if (Setup_Pkt->bRequest ==mGET_STATUS)
    {
 //           gu16Status=0;
            EP_IN_Transfer(EP0,(UINT8*)&gu16Status,2);
    }
    else
    {
            USB_EP0_Stall();                              
    }
}



/**********************************************************/
void USB_Setup_Handler(void)@".usb_hid_icp"
{
 //   UINT8 u8State;
    
    FLAG_CLR(0,gu8USB_Toogle_flags);
    
    if ((Setup_Pkt->bmRequestType & 0x7C)==0)                   // standard request
    {
        FLAG_SET(USB_CTL_ODDRST_SHIFT,USB0_CTL);  //derek
        
//        if (Setup_Pkt->bmRequestType & 0x03 == 0x02)
//          USB_Endpoint_Setup_Handler();
//        else
          USB_StdReq_Handler();            
        
        tBDTtable[bEP0OUT_ODD].Stat._byte= kUDATA0;
    }     
    

    else
    {
          FLAG_SET(USB_CTL_ODDRST_SHIFT,USB0_CTL);  //derek
          USB_InterfaceReq_Handler();    
          if (((Setup_Pkt->bLength_l)==0)||((Setup_Pkt->bmRequestType & 0x80)==0x80)){

                tBDTtable[bEP0OUT_ODD].Stat._byte= kUDATA0;

          }
    }         

        
    FLAG_CLR(USB_CTL_TXSUSPENDTOKENBUSY_SHIFT,USB0_CTL);
}




/**********************************************************/
void USB_Handler(void)
{
    UINT8 u8EndPoint;
    UINT8 u8IN;
    
    u8IN=USB0_STAT & 0x08;
    u8EndPoint=USB0_STAT >> 4;
    
    /* Data EndPoints */
    if((u8EndPoint)&&(u8IN==0))
    {
             usbMCU_CONTROL(u8EndPoint);
            FLAG_SET(u8EndPoint,gu8USB_Flags);  
     }


    /* Control EndPoint */
    else
    {
        if(u8IN)
            USB_EP0_IN_Handler();        
        else
        {
           if((tBDTtable[bEP0OUT_ODD].Stat.RecPid.PID == mOUT_TOKEN) ||
             (tBDTtable[bEP0OUT_EVEN].Stat.RecPid.PID == mOUT_TOKEN))
 
                USB_EP0_OUT_Handler();
        } 

          
          if(tBDTtable[bEP0OUT_ODD].Stat.RecPid.PID == mSETUP_TOKEN)
 
                USB_Setup_Handler();

 

    }
}

/**********************************************************/
void USB_EP0_IN_Handler(void)
{
    if(gu8USB_State==uADDRESS)
    {
        USB0_ADDR = Setup_Pkt->bValue_l;
        gu8USB_State=uREADY;
        FLAG_SET(fIN,gu8USBClearFlags);
    }
  
}


/**********************************************************/
void USB_EP0_Stall(void)
{
   
    FLAG_SET(USB_ENDPT_EPSTALL_SHIFT,USB0_ENDPT0);
            
    tBDTtable[bEP0OUT_ODD].Stat._byte = kUDATA0; 
    tBDTtable[bEP0OUT_ODD].Cnt = EP0_SIZE;       
}


/**********************************************************/
void USB_EP0_OUT_Handler(void)
{

  
  if (tBDTtable[bEP0OUT_ODD].Cnt != 0)
  {
    ICP_Handler((BootloaderCommand*)gu8EP0_OUT_ODD_Buffer);

  }
 
    tBDTtable[bEP0OUT_ODD].Stat._byte = kUDATA0;
  
}


/**********************************************************/
void USB_Stall_Handler(void) 
{
    if(FLAG_CHK(USB_ENDPT_EPSTALL_SHIFT,USB0_ENDPT0))
        FLAG_CLR(USB_ENDPT_EPSTALL_SHIFT,USB0_ENDPT0);
    FLAG_SET(USB_ISTAT_STALL_SHIFT,USB0_ISTAT);
}


/**********************************************************/
void USB_Reset_Handler(void)
{
    //PMC_VLPR_Exit();
      
    /* Software Flags */
    gu8USBClearFlags=0xFF;
    gu8USB_Toogle_flags=0;
    gu8USB_PingPong_flags=0x00;
    
    /* Disable all data EP registers */
    USB0_ENDPT1=0x00;

    /* EP0 BDT Setup */
    // EP0 OUT BDT Settings
    tBDTtable[bEP0OUT_ODD].Cnt = EP0_SIZE;
    tBDTtable[bEP0OUT_ODD].Addr =(UINT32)gu8EP0_OUT_ODD_Buffer;
    tBDTtable[bEP0OUT_ODD].Stat._byte = kUDATA1;         
    // EP0 OUT BDT Settings 
//    tBDTtable[bEP0OUT_EVEN].Cnt = EP0_SIZE;
//    tBDTtable[bEP0OUT_EVEN].Addr =(UINT32)gu8EP0_OUT_EVEN_Buffer;
//    tBDTtable[bEP0OUT_EVEN].Stat._byte = kUDATA0;         
    // EP0 IN BDT Settings 
    tBDTtable[bEP0IN_ODD].Cnt = EP0_SIZE;              
    tBDTtable[bEP0IN_ODD].Addr =(UINT32)gu8EP0_IN_ODD_Buffer;      
    tBDTtable[bEP0IN_ODD].Stat._byte = kUDATA1;   
    // EP0 IN BDT Settings 
//    tBDTtable[bEP0IN_EVEN].Cnt = EP0_SIZE;              
//    tBDTtable[bEP0IN_EVEN].Addr =(UINT32)gu8EP0_IN_EVEN_Buffer;      
//    tBDTtable[bEP0IN_EVEN].Stat._byte = kUDATA0;        

   // EP0 IN BDT Settings 
    tBDTtable[bEP1IN_ODD].Cnt = EP1_SIZE;              
    tBDTtable[bEP1IN_ODD].Addr =(UINT32)gu8EP0_IN_ODD_Buffer;      
    tBDTtable[bEP1IN_ODD].Stat._byte = kUDATA1;   
    // EP0 IN BDT Settings 
//    tBDTtable[bEP1IN_EVEN].Cnt = EP1_SIZE;              
//    tBDTtable[bEP1IN_EVEN].Addr =(UINT32)gu8EP0_IN_EVEN_Buffer;      
//    tBDTtable[bEP1IN_EVEN].Stat._byte = kUDATA0;        
    
  
    // Enable EP0
    USB0_ENDPT0=0x0D;

    // Clear all Error flags
    USB0_ERRSTAT=0xFF;
    
    // CLear all USB ISR flags
    USB0_ISTAT=0xFF;

    // Set default Address
    USB0_ADDR=0x00;

    // Enable all error sources
    USB0_ERREN=0xFF;

 
}


/**********************************************************/
void USB_Polling(void)
{
  
    if(FLAG_CHK(USB_ISTAT_USBRST_SHIFT,USB0_ISTAT)) 
    {
         USB_Reset_Handler();
        return;
    }
    
 /*   
    if(FLAG_CHK(USB_ISTAT_SOFTOK_SHIFT,USB0_ISTAT)) 
    {
        USB0_ISTAT = USB_ISTAT_SOFTOK_MASK;   
    }
 
    
    if(FLAG_CHK(USB_ISTAT_STALL_SHIFT,USB0_ISTAT)) 
    {
        USB_Stall_Handler();
    }
*/    
    
     if(FLAG_CHK(USB_ISTAT_TOKDNE_SHIFT,USB0_ISTAT)) 
    {

        USB_Handler();
//        FLAG_SET(USB_ISTAT_TOKDNE_SHIFT,USB0_ISTAT);
        FLAG_WR_CLR(USB_ISTAT_TOKDNE_SHIFT,USB0_ISTAT);
    }
    
    
    if(FLAG_CHK(USB_ISTAT_SLEEP_SHIFT,USB0_ISTAT)) 
    {
            // Clear RESUME Interrupt if Pending 
        //INT_STAT = INT_STAT_RESUME_MASK;
          //u8ISRCounter++;  
      //FLAG_SET(USB_ISTAT_RESUME_SHIFT,USB_ISTAT);
        
        // Clear SLEEP Interrupt 
//        FLAG_SET(USB_ISTAT_SLEEP_SHIFT,USB0_ISTAT);
        FLAG_WR_CLR(USB_ISTAT_SLEEP_SHIFT,USB0_ISTAT);
      //INT_STAT = INT_STAT_SLEEP_MASK;
        //FLAG_SET(USB_INTEN_RESUME_SHIFT,USB_ISTAT);
        //INT_ENB_RESUME_EN = 1;
        
    }
/*    
    if(FLAG_CHK(USB_ISTAT_ERROR_SHIFT,USB0_ISTAT)) 
    {
        FLAG_CHK(USB_ISTAT_ERROR_SHIFT,USB0_ISTAT);
        printf("\nUSB Error\n");
        //INT_STAT_ERROR=1;

    }
*/
}