#ifndef __USB_Desc__
#define __USB_Desc__

/* Includes */
#include "FslTypes.h"


/********************************************************************
*       Device Descriptor
********************************************************************/
const UINT8 Device_Descriptor[18]= 
{
	0x12,		    //blength
	0x01,		    //bDescriptor
	0x01,0x01,          //bcdUSB ver R=1.10 
	0x00,		    //bDeviceClass
	0x00,		    //bDeviceSubClass			
	0x00,		    //bDeviceProtocol			
	0x40,		    //bMaxPacketSize0
	0xA2,0x15,	    //idVendor - 0x15A2(freescale Vendor ID)
	0x03,0xFF,	    //idProduct
	0x00,0x01,	    //bcdDevice - Version 1.00
	0x01,		    //iManufacturer - Index to string Manufacturer descriptor
	0x02,		    //iProduct  - Index to string product descriptor 
	0x03,		    //iSerialNumber - Index to string serial number 
	0x01	            //bNumConfigurations - # of config. at current speed,
};


/********************************************************************
*       String 0
********************************************************************/
const UINT8 String_Descriptor0[4] = 
{
	0x04,		   //bLength;
	0x03,		   //bDescriptorType - STRING descriptor
	0x09,0x04	   //wLANDID0 - English (American)
};

/********************************************************************
*       String 1
********************************************************************/
const UINT8 String_Descriptor1[] = 
{
	8,		  //bLength; string descriptor bytes 
	0x03,		  //bDescriptorType - STRING descriptor
	'F',0x00,	  // "F"                                                                          
	's',0x00,	  // "r"
	'l',0x00,	  // "e"
};	 


/********************************************************************
*       String 2
********************************************************************/
const UINT8 String_Descriptor2[] = 
{
	8,			//bLength;
	0x03,		    //bDescriptorType - STRING descriptor
	'H',0x00,	    // HID
	'I',0x00,	
	'D',0x00,	
};	 
/********************************************************************
*       String 23
********************************************************************/
const UINT8 String_Descriptor3[] = 
{
	8,		//bLength;
	0x03,		//bDescriptorType - STRING descriptor
	'I',0x00,	// ICP
	'C',0x00,	
	'P',0x00,	
};	 


/*USB configuration descriptor structure*/
typedef struct _USB_CFG_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bTotalLengthL;
    UINT8 bTotalLengthH;
    UINT8 bNumIntf;
    UINT8 bCfgValue;
    UINT8 iCfg;
    UINT8 bmAttributes;
    UINT8 bMaxPower;
} USB_CFG_DSC;

/* USB interface descriptor structure*/
typedef struct _USB_INTF_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bIntfNum;
    UINT8 bAltSetting;
    UINT8 bNumEPs;
    UINT8 bIntfCls;
    UINT8 bIntfSubCls;
    UINT8 bIntfProtocol;
    UINT8 iIntf;
} USB_INTF_DSC;

/*USB endpoint descriptor structure*/
typedef struct _USB_EP_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bEPAdr;
    UINT8 bmAttributes;
    UINT8 bMaxPktSizeL;
    UINT8 bMaxPktSizeH;
    UINT8 bInterval;
} USB_EP_DSC;

typedef struct _USB_HID_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bcdHID_L;
    UINT8 bcdHID_H;
    UINT8 bCountryCode;
    UINT8 bNumDsc;
    UINT8 bRptDecType;
    UINT8 bItemLen_L;
    UINT8 bITEMLen_H;
} USB_HID_DSC;

typedef struct _USB_CFG                       
{   
    USB_CFG_DSC     Cfg_Dsc_01;                  
    USB_INTF_DSC    Int_Dsc_Intf00;                
    USB_HID_DSC     Hid_Dsc_Intf00;           
    USB_EP_DSC      Ep1IN_Dsc_Intf00;
}USB_CFG;



/********************************************************************
*       Configuration Descriptor
********************************************************************/
const USB_CFG Configuration_Descriptor[]= 
{
	
	0x09,		    //blength
	0x02,		    //bDescriptor
	9+9+9+7,0x00,	    //wTotalLength - # of bytes including interface and endpoint descpt.
	0x01,		    //bNumInterfaces - at least 1 data interface
	0x01,		    //bConfigurationValue - 
	0x00,		    //iConfiguration - index to string descriptor	
	0xC0,		    //bmAttributes - 	            bit 7- Compatibility with USB 1.0
				    //						        bit 6 if 1 self powered else Bus powered
				    //						        bit 5-remote wakeup
				    //						        bit 4-0-reserved
	0x32,		    //bMaxPower - 200mA
	
/********************************************************************
*       Interface Descriptor
********************************************************************/

	0x09,	        //blength
	0x04,		//bDescriptorType - Interface descriptor
	0x00,		//bInterfaceNumber - Zero based value identifying the index of the config.
	0x00,		//bAlternateSetting;
	0x01,		//bNumEndpoints - 1 endpoints
	0x03,		//bInterfaceClass - HID Class
	0x00,		//bInterfaceSubClass - not boot interface
	0x00,		//bInterfaceProtocol - NA
	0x00,		//iInterface - Index to String descriptor

/********************************************************************
*       HID Descriptor
********************************************************************/

        9,
        0x21,           // HID Class            
        0x10,0x01,      // HID Class spec ver 1.10
        0x00,           // country code = 00
        0x01,           // no of HID descriptor follow
        0x22,           // report descriptor
        37,0x00,      // report descritpor length
      
/********************************************************************
*       Endpoint  Descriptor
********************************************************************/
	0x07,           //blength
	0x05,           //bDescriptorType - EndPoint
	0x81,           //bEndpointAddress    
	0x03,           //bmAttributes
	0x40,0x00,      //wMaxPacketSize           
	0x01,           //bInterval
};



/********************************************************************
*       Device Qualifier Descriptor
********************************************************************/
const UINT8 Device_Qualifier[10]=
{
    0x0A,
    0x06,
    0x00,0x02,
    0x00,
    0x00,
    0x00,
    0x10,
    0x01,
    0x00
};






#endif /* __USB_Desc__ */