#ifndef __USBICP__
#define __USBICP__

/* Includes */
#include "common.h"
#include  "USB.h"
#include  "FslTypes.h"
#include  "RingBuffer.h"


//#define CUSTOM_BASE_ADDRESS 0xC000  /* Base Address for Customizable settings */

/* USB_ICP Definitions */

#define ICP_PASS                  0x00;
#define ICP_FAIL                  0xFF;
#define ICP_EP0_SIZE              64        // must equal ep0 size
#define ICP_STATUS_IDLE           0x00
#define ICP_STATUS_BEGIN          0x01
#define BL_COMMAND_ERASE          0x01
#define BL_COMMAND_PROGRAM_BYTE   0x11
#define BL_COMMAND_PROGRAM_BLOCK  0x12
#define BL_COMMAND_FILL_BLOCK     0x13
#define BL_COMMAND_GET_STATUS     0x81

#define USE_CHECKSUM              1      
#if (defined(PFLASH_64K))
 #define USER_ADDR              0x1000 
 #define APPCHECKSUM_ADDR         0x1400
  
#elif (defined(PFLASH_128K))
 #define USER_ADDR                0x1000
 #define APPCHECKSUM_ADDR         0x1400

#elif (defined(PFLASH_256K))
  #define USER_ADDR               0x2000
  #define APPCHECKSUM_ADDR        0x2400

  
#elif (defined(PFLASH_512K))
  #define USER_ADDR               0x4000
  #define APPCHECKSUM_ADDR        0x4400

#else
  #error "No FPlash size is defined, eg: PFLASH 512K"
#endif

#define APPCHECKSUM32        ((UINT32*)APPCHECKSUM_ADDR)
#define APPCHECKSUM8        ((UINT8*)APPCHECKSUM_ADDR)

/* TypeDefs */


/* Extern variables */



/* Prototypes */
#pragma pack(2)
typedef union _BootloaderCommand
{
    struct
    {
        UINT8 BootloaderID;           //0
        UINT8 Command;          //1
        UINT8 _byte[ICP_EP0_SIZE-2];
    }C;

    struct
    {
         UINT8 *Data;
    }Data;

    
    __packed struct
    {
        UINT8 BootloaderID;       //0
        UINT8 Command;            //1
        UINT32 FlashAddress;      //2-5
        UINT16 Length;            //6-7
        UINT8 Data[ICP_EP0_SIZE-8];
    }ProgramByte;

    __packed struct
    {
        UINT8 BootloaderID;     //0
        UINT8 Command;          //1
        UINT32 FlashAddress;    //2-5
    }Erase;

   __packed struct
    {
        UINT8 BootloaderID;       //0
        UINT8 Command;            //1
        UINT32 FlashAddress;      //2-5
        UINT16 Length;            //6-7
     }ProgramBlock;
    
  __packed struct
    {
        UINT8 BootloaderID;       //0
        UINT8 Command;            //1
        UINT32 BufferPointer;      //2-5
        UINT16 Length;            //6-7
        UINT8 Data[ICP_EP0_SIZE-8];
    }FillBlock;
     
    
} BootloaderCommand;


void ICP_FillBuffer(UINT8 *Data);
void ICP_Handler(BootloaderCommand *BLBuffer);

#endif /* __USBICP__*/