#ifndef __USB_HID__
#define __USB_HID__

/* Includes */
#include"USB.h"
#include "FslTypes.h"
#include "RingBuffer.h"



//#define CUSTOM_BASE_ADDRESS 0xC000  /* Base Address for Customizable settings */




/* Defines */




/* USB_HID Definitions */
#define USB_HID_GET_REPORT_REQUEST     (0x01)
#define USB_HID_GET_IDLE_REQUEST       (0x02)
#define USB_HID_GET_PROTOCOL_REQUEST   (0x03)
#define USB_HID_SET_REPORT_REQUEST     (0x09)
#define USB_HID_SET_IDLE_REQUEST       (0x0A)
#define USB_HID_SET_PROTOCOL_REQUEST   (0x0B)

#define USB_HID_GET_HID_DESC           (0x21)
#define USB_HID_GET_REPORT_DESC        (0x22)
#define USB_HID_GET_PHY_DESC           (0x23)


#define USB_REQUEST_DATADIR(val)      (val & 0x80)
#define USB_REQUEST_TYPE(val)         (val & 0x60)
#define USB_REQUEST_RECIPIENT(val)    (val & 0x1F)

#define USB_HID_DESCRIPTOR            (0x81)


/* TypeDefs */
typedef struct
{
    UINT32  DTERate;
    UINT8   CharFormat;
    UINT8   ParityType;
    UINT8   Databits;
}CDC_Line_Coding;

/*USB configuration descriptor structure*/
typedef struct _USB_CFG_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bTotalLengthL;
    UINT8 bTotalLengthH;    
    UINT8 bNumIntf;
    UINT8 bCfgValue;
    UINT8 iCfg;
    UINT8 bmAttributes;
    UINT8 bMaxPower;
} USB_CFG_DSC;

/* USB interface descriptor structure*/
typedef struct _USB_INTF_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bIntfNum;
    UINT8 bAltSetting;
    UINT8 bNumEPs;
    UINT8 bIntfCls;
    UINT8 bIntfSubCls;
    UINT8 bIntfProtocol;
    UINT8 iIntf;
} USB_INTF_DSC;

/*USB endpoint descriptor structure*/
typedef struct _USB_EP_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bEPAdr;
    UINT8 bmAttributes;
    UINT8 bMaxPktSizeL;
    UINT8 bMaxPktSizeH;
    UINT8 bInterval;
} USB_EP_DSC;

typedef struct _USB_HID_DSC
{
    UINT8 bLength;
    UINT8 bDscType;
    UINT8 bcdHID_L;
    UINT8 bcdHID_H;
    UINT8 bCountryCode;
    UINT8 bNumDsc;
    UINT8 bRptDecType;
    UINT8 bItemLen_L;
    UINT8 bITEMLen_H;
} USB_HID_DSC;

typedef struct _USB_CFG                       
{   
    USB_CFG_DSC     Cfg_Dsc_01;                  
    USB_INTF_DSC    Int_Dsc_Intf00;                
    USB_HID_DSC     Hid_Dsc_Intf00;           
    USB_EP_DSC      Ep1IN_Dsc_Intf00;
}USB_CFG;


/* Extern variables */
    
//extern UINT8 u8RecDataFlag;        


// ======================================================
//
// 		Standard Report Descriptor
//
// ======================================================
#define  kMtagInput     0x80  
#define  kMtagOutput    0x90  
#define  kMtagFeature   0xB0  
#define  kMtagCollectn  0xA0   
#define  kMtagEndCltn   0xC0  

#define  kGtagUPage     0x04     
#define  kGtagLogMin    0x14   
#define  kGtagLogMax    0x24     
#define  kGtagPhyMin    0x34     
#define  kGtagPhyMax    0x44    
#define  kGtagUnitExp   0x54    
#define  kGtagUnit      0x64    
#define  kGtagRptSize   0x74   
#define  kGtagRptID     0x84    
#define  kGtagRptCnt    0x94   
#define  kGtagPush      0xA4   
#define  kGtagPop       0xB4  

#define  kGtagUPage     0x04     
#define  kGtagLogMin    0x14   
#define  kGtagLogMax    0x24   
#define  kGtagPhyMin    0x34  
#define  kGtagPhyMax    0x44 
#define  kGtagUnitExp   0x54   
#define  kGtagUnit      0x64   
#define  kGtagRptSize   0x74    
#define  kGtagRptID     0x84    
#define  kGtagRptCnt    0x94  
#define  kGtagPush      0xA4
#define  kGtagPop       0xB4     

#define  kLtagUsage     0x08    
#define  kLtagUsgMin    0x18    
#define  kLtagUsgMax    0x28    
#define  kLtagDgntrIdx  0x38    
#define  kLtagDgntrMin  0x48    
#define  kLtagDgntrMax  0x58    
#define  kLtagStrgIdx   0x78    
#define  kLtagStrgMin   0x88     
#define  kLtagStrgMax   0x98    
#define  kLtagSDlimitr  0xA8     


const UINT8 ICP_Report_Descriptor[] = {
  kGtagUPage|0x02,0x00,0xFF,  // Usage Page(Vendor Define)
  kLtagUsage|0x01,0x01,       // Usage(Vendor Control)
  kMtagCollectn|0x01,0x01,    // Collection(Application)
  kGtagLogMin|0x01,0x00,      // Logical Min(0)
  kGtagLogMax|0x02,0xFF,0x00, // Logical Max(255)
  kGtagRptSize|0x01,0x08,     // Report Size(8)
  kGtagRptCnt|0x01,EP1_SIZE,        // Report Count(64)
  kLtagUsgMin|0x01,0x01,      // Usage min
  kLtagUsgMax|0x01,0x40,      // Usage max
  kMtagInput|0x01,0x02,       // Input (Data,Variable)
  kLtagUsage|0x01,0x01,       // Usage(Byte Count)

  kGtagRptSize|0x01,0x08,     // Report Size(8)
  kGtagRptCnt|0x01,EP0_SIZE,        // Report Count(64)
  kLtagUsgMin|0x01,0x01,      // Usage max
  kLtagUsgMax|0x01,0x40,      // Usage max
  kMtagOutput|0x01,0x02,      // Output(Data,Variable)
  kLtagUsage|0x01,0x01,       // Usage(Byte Count)

  kMtagEndCltn                // End_Collection
};

/* Prototypes */
void CDC_Init(void);
void CDC_Engine(void);
UINT8 HID_InterfaceReq_Handler(void);
void HID_GetRptHandler(void);
void HID_SetRptHandler(void);
UINT32 LWordSwap(UINT32);
UINT8 CDC_Settings_Update(UINT8*,UINT8);


#endif /* __USB_HID__*/