#include "common.h"
#include "uif.h"
#include "SSD_Types.h"
#include "SSD_FTFL.h"
#include "SSD_FTFL_Internal.h"
#include "USB_ICP.h"

#define FLASH_CMD_SIZE            0x200

// Flash Driver configuration structure
FLASH_SSD_CONFIG flashSSDConfig =
{
    (uint32)FTFL_BASE_PTR,  /* FTFx control register base */
    PFLASH_BLOCK_BASE,      /* base address of PFlash block */
    PBLOCK_SIZE,            /* size of PFlash block */
    DEFLASH_BLOCK_BASE,     /* base address of DFlash block */
    DEBLOCK_SIZE,           /* size of DFlash block */
    EERAM_BLOCK_BASE,       /* base address of EERAM block */
    EERAM_BLOCK_SIZE,       /* size of EERAM block */
    0,                      /* size of EEE block */
    0,            /* background debug mode enable bit */
    NULL_CALLBACK           /* pointer to callback function */
};

UINT8 flashCmd[FLASH_CMD_SIZE];
pFLASHCOMMANDSEQUENCE pFlashCommandSequence;

UINT8 Flash_Erase_Sector(UINT32 addr)
{
    UINT32 returnCode;
    
    returnCode = FlashEraseSector(&flashSSDConfig, addr, \
                                FTFL_SECTOR_SIZE, FlashCommandSequence);

      if (returnCode==0)
      {
        returnCode = FlashVerifySection(&flashSSDConfig, addr, FTFL_SECTOR_SIZE/FTFL_PHRASE_SIZE, 0, FlashCommandSequence);
      }
    
    return returnCode;
}


// Setup the flash driver
void Init_Flash_Driver(void)
{
    UINT32 flashCmdFr;
    UINT32 flashCmdTo;
    UINT32 i;
    UINT8 value;
    

    // Copy the FlashCommandSequence to RAM
    flashCmdFr = (UINT32)FlashCommandSequence - 1;
    flashCmdTo = (UINT32)&flashCmd;
    for(i=0;i < FLASH_CMD_SIZE;i++)
    {
       value = READ8(flashCmdFr);
       flashCmdFr++;
       WRITE8(flashCmdTo, value);
       flashCmdTo++;
    }
    
    // Load pointer to command sequence in RAM
    pFlashCommandSequence = (pFLASHCOMMANDSEQUENCE)(((UINT32)&flashCmd) + 1);
    
    // Initialize the Flash Driver
//    if(FlashInit(&flashSSDConfig))
//        for(;;);        // Error in FlashInit
    
    
}
