/******************************************************************************
 *
 * Freescale Semiconductor Inc.
 * (c) Copyright 2012-2013 Freescale Semiconductor, Inc.
 * ALL RIGHTS RESERVED.
 *
 ******************************************************************************
 *
 * @file usb.c
 *
 * @author
 *
 * @version
 *
 * @date 2013-06-14
 *
 * @brief  
 *
 *
 *****************************************************************************
 * Includes
 *****************************************************************************/
#include "USB.h"
#include "USB_Desc.h"
#include "usb_icp.h"

 /*****************************************************************************
 * Constant and Macro's - None
 *****************************************************************************/

/*****************************************************************************
 * Global Functions Prototypes
 *****************************************************************************/
extern uint_8 HID_InterfaceReq_Handler(void);
            
/****************************************************************************
 * Global Variables 
 ****************************************************************************/
/* Arrays and global buffers */
#pragma data_alignment=512
tBDT tBDTtable[8];

uint_8 gu8EP0_OUT_ODD_Buffer[EP0_SIZE];
uint_8 gu8EP0_OUT_EVEN_Buffer[EP0_SIZE];
uint_8 gu8EP0_IN_ODD_Buffer[EP0_SIZE];
uint_8 gu8EP0_IN_EVEN_Buffer[EP0_SIZE];
uint_8 gu8EP1_OUT_ODD_Buffer[EP1_SIZE];
uint_8 gu8EP1_OUT_EVEN_Buffer[EP1_SIZE];
uint_8 gu8EP1_IN_ODD_Buffer[EP1_SIZE];
uint_8 gu8EP1_IN_EVEN_Buffer[EP1_SIZE];

uint_8 *BufferPointer[]=
{
    gu8EP0_OUT_ODD_Buffer,
    gu8EP0_OUT_EVEN_Buffer,
    gu8EP0_IN_ODD_Buffer,
    gu8EP0_IN_EVEN_Buffer,
    gu8EP1_OUT_ODD_Buffer,
    gu8EP1_OUT_EVEN_Buffer,
    gu8EP1_IN_ODD_Buffer,
    gu8EP1_IN_EVEN_Buffer,
 
};

const uint_8 cEP_Size[]=
{
    EP0_SIZE,    
    EP0_SIZE,    
    EP0_SIZE,    
    EP0_SIZE,    
    EP1_SIZE,    
    EP1_SIZE,    
    EP1_SIZE,    
    EP1_SIZE,    
 };

const uint_8* String_Table[4]=
{
    String_Descriptor0,
    String_Descriptor1,
    String_Descriptor2,
    String_Descriptor3
};

/* Global Variables for USB Handling */
uint_8 gu8USB_Flags;
uint_8 gu8USBClearFlags;
uint_8 *pu8IN_DataPointer;
uint_8 gu8IN_Counter;
uint_8 gu8USB_Toogle_flags;
uint_8 gu8USB_PingPong_flags;
uint_8 gu8USB_Configure;
uint_8 gu8USB_Interface;
uint_16 gu16Status;
uint_8 gu8Interface;
uint_8 gu8HALT_EP;
uint_8 gu8USB_State;
tUSB_Setup *Setup_Pkt;
 

/*****************************************************************************
 * Local Types - None
 *****************************************************************************/

/*****************************************************************************
 * Local Functions Prototypes (called by external routines)
 *****************************************************************************/   
void USB_Init(void);

/**********************************************************/
void USB_Init(void)
{  
    /* Software Configuration */
    Setup_Pkt=(tUSB_Setup*)BufferPointer[bEP0OUT_ODD];   // assign setup packet address
    gu8USB_State=uPOWER;                                 // power stage

    /* MPU Configuration */
//    MPU_CESR=0;                                        // MPU is disable. All accesses from all bus masters are allowed

    //clock gate and USB clock source
    SIM_SOPT2 |= SIM_SOPT2_USBSRC_MASK;   // Set USB clock source to PLL/2
    SIM_SCGC4 |= SIM_SCGC4_USBOTG_MASK;   // Enable clock gate for USB

 
     /* USB Module Configuration */
    // Reset USB Module
    USB0_USBTRC0|=USB_USBTRC0_USBRESET_MASK;
    while(USB0_USBTRC0 & USB_USBTRC0_USBRESET_MASK){};
    
    // Set BDT Base Register in MCU registers
    USB0_BDTPAGE1=(uint_8)((uint_32)tBDTtable>>8);
    USB0_BDTPAGE2=(uint_8)((uint_32)tBDTtable>>16);
    USB0_BDTPAGE3=(uint_8)((uint_32)tBDTtable>>24);

    USB0_ISTAT = 0xFF;    // clear any flags
    USB0_INTEN = 0x00;    // disable all interrupts
    USB0_OTGICR = 0x00;   // disable all OTG interrupts
    USB0_ERREN = 0x00;    // disable all error interrupts
    
    // Enable weak pull downs
    USB0_USBCTRL = USB_USBCTRL_PDE_MASK;

    USB0_USBTRC0|=0x40;                 // bit 6 must be 1
    USB0_CTL|=USB_CTL_USBENSOFEN_MASK;  // usb module enable
    
  //enable USB regulator, and pullup resistor
    SIM_SOPT1 |= SIM_SOPT1_USBREGEN_MASK;         // Enable USB regulator
    USB0_OTGCTL =  USB_OTGCTL_DPHIGH_MASK         // Enable DP pull-up, starts enumeration
                |  USB_OTGCTL_OTGEN_MASK;
}

/**********************************************************/
void EP_IN_Transfer(uint_8 u8EP,uint_8 *pu8DataPointer,uint_8 u8DataSize)
{
    uint_8 *pu8EPBuffer;
    uint_8 u8EPSize; 
    uint_16 u16Length=0;    
    uint_8 u8EndPointFlag;    

    /* Adjust the buffer location */
    u8EndPointFlag=u8EP;
//    if(u8EP)
        u8EP=(uint_8)(u8EP<<2);              
        u8EP+=2;                     // u8Ep (buffer location) = (u8EP*2)+2, eg: EP0In <-> bufferPointer[2]

    /* Assign the proper EP buffer */
    pu8EPBuffer=BufferPointer[u8EP];
    
    /* Check if is a pending transfer */
    if(FLAG_CHK(fIN,gu8USBClearFlags))
    {
        pu8IN_DataPointer=pu8DataPointer;
        gu8IN_Counter=u8DataSize;       

        u16Length=(Setup_Pkt->bLength_h<<8)+Setup_Pkt->bLength_l ;
        if((u16Length < u8DataSize) && (u8EP==2))
        {
            gu8IN_Counter=Setup_Pkt->bLength_l;
        }
    }

    /* Check transfer Size */
//    if(gu8IN_Counter > cEP_Size[u8EP])
//    {
//        u8EPSize = cEP_Size[u8EP];
//        gu8IN_Counter-=cEP_Size[u8EP];
//        FLAG_CLR(fIN,gu8USBClearFlags);
//    }
//    else
//    {
        u8EPSize = gu8IN_Counter;
        gu8IN_Counter=0;
        FLAG_SET(fIN,gu8USBClearFlags);
//    }
    
    /* Copy User buffer to EP buffer */
    tBDTtable[u8EP].Cnt=(u8EPSize);
    
    while(u8EPSize--)
         *pu8EPBuffer++=*pu8IN_DataPointer++;
                                                             

    /* USB Flags Handling */
    if(FLAG_CHK(u8EndPointFlag,gu8USB_Toogle_flags))
    {
        tBDTtable[u8EP].Stat._byte= kUDATA0;
        FLAG_CLR(u8EndPointFlag,gu8USB_Toogle_flags);
    } 
    else
    {
        tBDTtable[u8EP].Stat._byte= kUDATA1;
        FLAG_SET(u8EndPointFlag,gu8USB_Toogle_flags);
    }

}


/**********************************************************/
uint_8 EP_OUT_Transfer(uint_8 u8EP,uint_8 *pu8DataPointer)
{
    uint_8 *pu8EPBuffer;
    uint_8 u8EPSize; 
    

    /* Adjust the buffer location */
    u8EP = u8EP << 1;               // need this code for other Endpoints, (skip for EP0 only)
    u8EP++;                        // u8Ep (buffer location) = (u8EP*2)+1, eg: EP0Out <-> bufferPointer[1] 

    /* Assign the proper EP buffer */
    pu8EPBuffer=BufferPointer[u8EP];
    
    /* Copy User buffer to EP buffer */
    u8EPSize=tBDTtable[u8EP].Cnt;
    u8EP=u8EPSize;
    
    while(u8EPSize--)
         *pu8DataPointer++=*pu8EPBuffer++;
    return(u8EP);
}

/**********************************************************/
uint_16 USB_EP_OUT_SizeCheck(uint_8 u8EP)
{
    uint_8 u8EPSize; 
    /* Read Buffer Size */
    u8EPSize=SWAP16(tBDTtable[u8EP<<2].Cnt);
    return(u8EPSize&0x03FF);
}

/**********************************************************/
void USB_Set_Interface(void)
{
    /* EndPoint Register settings */
    USB0_ENDPT1= EP1_VALUE | USB_ENDPT_EPHSHK_MASK;                         
 

    /* EndPoint 1 BDT Settings*/
    tBDTtable[bEP1IN_ODD].Stat._byte= kMCU;
    tBDTtable[bEP1IN_ODD].Cnt = 0x00;
    tBDTtable[bEP1IN_ODD].Addr =(uint_32)gu8EP1_IN_ODD_Buffer;

         
}

/**********************************************************/
void USB_StdReq_Handler(void)
{

    if (Setup_Pkt->bRequest ==mSET_ADDRESS )
    {
             EP_IN_Transfer(EP0,0,0);
            gu8USB_State=uADDRESS;
    }
      
    else if  (Setup_Pkt->bRequest ==mGET_DESC)
    {
            if (Setup_Pkt->bValue_h == mDEVICE) 
            {
               EP_IN_Transfer(EP0,(uint_8*)Device_Descriptor,sizeof(Device_Descriptor));
            }
                    
            else if(Setup_Pkt->bValue_h == mCONFIGURATION)
            {
 //                   EP_IN_Transfer(EP0,(uint_8*)Configuration_Descriptor,sizeof(Configuration_Descriptor));
                   EP_IN_Transfer(EP0,(uint_8*)Configuration_Descriptor,34);
            }
        
            else if (Setup_Pkt->bValue_h == mSTRING)
            {
                    EP_IN_Transfer(EP0,(uint_8*)String_Table[Setup_Pkt->bValue_l],String_Table[Setup_Pkt->bValue_l][0]);
            }

            else
            {  
               
              HID_InterfaceReq_Handler();

            }
    }

    else if (Setup_Pkt->bRequest ==mSET_CONFIG)
    {
           gu8USB_Configure=Setup_Pkt->bValue_h+Setup_Pkt->bValue_l;
            if(Setup_Pkt->bValue_h+Setup_Pkt->bValue_l) 
            {
                USB_Set_Interface();                         

                gu8USB_State=uENUMERATED;

            }
           EP_IN_Transfer(EP0,0,0);
    }
    
    else if (Setup_Pkt->bRequest ==mGET_CONFIG)
    {
            EP_IN_Transfer(EP0,(uint_8*)&gu8USB_Configure,1);
    }

   else if (Setup_Pkt->bRequest == mSET_INTF)
    {
           gu8USB_Interface=Setup_Pkt->bValue_h+Setup_Pkt->bValue_l;
                EP_IN_Transfer(EP0,0,0);
    }
      
    else if (Setup_Pkt->bRequest ==mGET_INTF)
    {
            EP_IN_Transfer(EP0,(uint_8*)&gu8USB_Interface,1);
    }
    
    else if (Setup_Pkt->bRequest ==mSET_FEATURE)
    {
            gu16Status = 0x0001;
            EP_IN_Transfer(EP0,0,0);
    }

    else if (Setup_Pkt->bRequest ==mCLR_FEATURE)
    {
            gu16Status = 0x0000;
            EP_IN_Transfer(EP0,0,0);
    }
    
    
    else if (Setup_Pkt->bRequest ==mGET_STATUS)
    {
 //           gu16Status=0;
            EP_IN_Transfer(EP0,(uint_8*)&gu16Status,2);
    }
    else
    {
            USB_EP0_Stall();                              
    }
}

/**********************************************************/
void USB_StdReq_Handler1(void)
{

    switch(Setup_Pkt->bRequest)
    {
        case mSET_ADDRESS:
            EP_IN_Transfer(EP0,0,0);
            gu8USB_State=uADDRESS;
            break;
      
        case mGET_DESC:
            switch(Setup_Pkt->bValue_h) 
            {
                case mDEVICE:
                    EP_IN_Transfer(EP0,(uint_8*)Device_Descriptor,sizeof(Device_Descriptor));
                    break;
                    
                case mCONFIGURATION:
 //                   EP_IN_Transfer(EP0,(uint_8*)Configuration_Descriptor,sizeof(Configuration_Descriptor));
                   EP_IN_Transfer(EP0,(uint_8*)Configuration_Descriptor,34);
 
                    break;
        
                case mSTRING:
                    EP_IN_Transfer(EP0,(uint_8*)String_Table[Setup_Pkt->bValue_l],String_Table[Setup_Pkt->bValue_l][0]);
                    break;

                default:
                    USB_EP0_Stall();
                    break;  
            }
            break;

        case mSET_CONFIG:
            gu8USB_Configure=Setup_Pkt->bValue_h+Setup_Pkt->bValue_l;
            if(Setup_Pkt->bValue_h+Setup_Pkt->bValue_l) 
            {
                USB_Set_Interface();                         
                EP_IN_Transfer(EP0,0,0);
                gu8USB_State=uENUMERATED;
            }
            break;
      
        case mGET_CONFIG:
            EP_IN_Transfer(EP0,(uint_8*)&gu8USB_Configure,1);
            break;

        case mGET_STATUS:
            gu16Status=0;
            EP_IN_Transfer(EP0,(uint_8*)&gu16Status,2);
            break;


        default:
            USB_EP0_Stall();                              
            break;
    }
}

/**********************************************************/
void USB_Setup_Handler(void)
{
  
    FLAG_CLR(0,gu8USB_Toogle_flags);
    
    if ((Setup_Pkt->bmRequestType & 0x7C)==0)                   // standard request
    {
        FLAG_SET(USB_CTL_ODDRST_SHIFT,USB0_CTL);  //derek
        
//        if (Setup_Pkt->bmRequestType & 0x03 == 0x02)
//          USB_Endpoint_Setup_Handler();
//        else
          USB_StdReq_Handler();            
        
        tBDTtable[bEP0OUT_ODD].Stat._byte= kUDATA0;
    }     
    

    else
    {
          FLAG_SET(USB_CTL_ODDRST_SHIFT,USB0_CTL);  //derek
          USB_InterfaceReq_Handler();    
          if (((Setup_Pkt->bLength_l)==0)||((Setup_Pkt->bmRequestType & 0x80)==0x80)){

                tBDTtable[bEP0OUT_ODD].Stat._byte= kUDATA0;

          }
    }         

        
    FLAG_CLR(USB_CTL_TXSUSPENDTOKENBUSY_SHIFT,USB0_CTL);
}




/**********************************************************/
void USB_Handler(void)
{
    uint_8 u8EndPoint;
    uint_8 u8IN;
    
    u8IN=USB0_STAT & 0x08;
    u8EndPoint=USB0_STAT >> 4;
    
    /* Data EndPoints */
    if((u8EndPoint)&&(u8IN==0))
    {
             usbMCU_CONTROL(u8EndPoint);
            FLAG_SET(u8EndPoint,gu8USB_Flags);  
     }


    /* Control EndPoint */
    else
    {
        if(u8IN)
            USB_EP0_IN_Handler();        
        else
        {
           if((tBDTtable[bEP0OUT_ODD].Stat.RecPid.PID == mOUT_TOKEN) ||
             (tBDTtable[bEP0OUT_EVEN].Stat.RecPid.PID == mOUT_TOKEN))
 
                USB_EP0_OUT_Handler();
        } 

          
          if(tBDTtable[bEP0OUT_ODD].Stat.RecPid.PID == mSETUP_TOKEN)
 
                USB_Setup_Handler();

 

    }
}

/**********************************************************/
void USB_EP0_IN_Handler(void)
{
    if(gu8USB_State==uADDRESS)
    {
        USB0_ADDR = Setup_Pkt->bValue_l;
        gu8USB_State=uREADY;
        FLAG_SET(fIN,gu8USBClearFlags);
    }
  
}


/**********************************************************/
void USB_EP0_Stall(void)
{
   
    FLAG_SET(USB_ENDPT_EPSTALL_SHIFT,USB0_ENDPT0);
            
    tBDTtable[bEP0OUT_ODD].Stat._byte = kUDATA0; 
    tBDTtable[bEP0OUT_ODD].Cnt = EP0_SIZE;       
}


/**********************************************************/
void USB_EP0_OUT_Handler(void)
{

  
  if (tBDTtable[bEP0OUT_ODD].Cnt != 0)
  {
    ICP_Handler((BootloaderCommand*)gu8EP0_OUT_ODD_Buffer);

  }
 
    tBDTtable[bEP0OUT_ODD].Stat._byte = kUDATA0;
  
}


/**********************************************************/
void USB_Stall_Handler(void) 
{
    if(FLAG_CHK(USB_ENDPT_EPSTALL_SHIFT,USB0_ENDPT0))
        FLAG_CLR(USB_ENDPT_EPSTALL_SHIFT,USB0_ENDPT0);
    FLAG_SET(USB_ISTAT_STALL_SHIFT,USB0_ISTAT);
}


/**********************************************************/
void USB_Reset_Handler(void)
{
    //PMC_VLPR_Exit();
      
    /* Software Flags */
    gu8USBClearFlags=0xFF;
    gu8USB_Toogle_flags=0;
    gu8USB_PingPong_flags=0x00;
    
    /* Disable all data EP registers */
    USB0_ENDPT1=0x00;

    /* EP0 BDT Setup */
    // EP0 OUT BDT Settings
    tBDTtable[bEP0OUT_ODD].Cnt = EP0_SIZE;
    tBDTtable[bEP0OUT_ODD].Addr =(uint_32)gu8EP0_OUT_ODD_Buffer;
    tBDTtable[bEP0OUT_ODD].Stat._byte = kUDATA1;         
    // EP0 OUT BDT Settings 
//    tBDTtable[bEP0OUT_EVEN].Cnt = EP0_SIZE;
//    tBDTtable[bEP0OUT_EVEN].Addr =(UINT32)gu8EP0_OUT_EVEN_Buffer;
//    tBDTtable[bEP0OUT_EVEN].Stat._byte = kUDATA0;         
    // EP0 IN BDT Settings 
    tBDTtable[bEP0IN_ODD].Cnt = EP0_SIZE;              
    tBDTtable[bEP0IN_ODD].Addr =(uint_32)gu8EP0_IN_ODD_Buffer;      
    tBDTtable[bEP0IN_ODD].Stat._byte = kUDATA1;   
    // EP0 IN BDT Settings 
//    tBDTtable[bEP0IN_EVEN].Cnt = EP0_SIZE;              
//    tBDTtable[bEP0IN_EVEN].Addr =(UINT32)gu8EP0_IN_EVEN_Buffer;      
//    tBDTtable[bEP0IN_EVEN].Stat._byte = kUDATA0;        

   // EP0 IN BDT Settings 
    tBDTtable[bEP1IN_ODD].Cnt = EP1_SIZE;              
    tBDTtable[bEP1IN_ODD].Addr =(uint_32)gu8EP0_IN_ODD_Buffer;      
    tBDTtable[bEP1IN_ODD].Stat._byte = kUDATA1;   
    // EP0 IN BDT Settings 
//    tBDTtable[bEP1IN_EVEN].Cnt = EP1_SIZE;              
//    tBDTtable[bEP1IN_EVEN].Addr =(UINT32)gu8EP0_IN_EVEN_Buffer;      
//    tBDTtable[bEP1IN_EVEN].Stat._byte = kUDATA0;        
    
  
    // Enable EP0
    USB0_ENDPT0=0x0D;

    // Clear all Error flags
    USB0_ERRSTAT=0xFF;
    
    // CLear all USB ISR flags
    USB0_ISTAT=0xFF;

    // Set default Address
    USB0_ADDR=0x00;

    // Enable all error sources
    USB0_ERREN=0xFF;

 
}


/**********************************************************/
void USB_Polling(void)
{
  
     
     if(FLAG_CHK(USB_ISTAT_TOKDNE_SHIFT,USB0_ISTAT)) 
    {

        USB_Handler();
        FLAG_WR_CLR(USB_ISTAT_TOKDNE_SHIFT,USB0_ISTAT);
    }
    
    else if(FLAG_CHK(USB_ISTAT_USBRST_SHIFT,USB0_ISTAT)) 
    {
         USB_Reset_Handler();
        return;
    }
    
   
    else if(FLAG_CHK(USB_ISTAT_SLEEP_SHIFT,USB0_ISTAT)) 
    {
        FLAG_WR_CLR(USB_ISTAT_SLEEP_SHIFT,USB0_ISTAT);
         
    }


}