#ifndef __USB__
#define __USB__


#include "FslTypes.h"
#include "common.h"
//#include "PMC.h"


/* Select Clock source */
#define USB_CLOCK_PLL
//#define USB_CLOCK_FLL
//#define USB_CLOCK_CLKIN

/* USB Fractional Divider Value */
#define USB_FARCTIONAL_VALUE    0x02




/*********** HIL ***************/
#include "USB_icp.h"
#define USB_InterfaceReq_Handler  HID_InterfaceReq_Handler   
/********************************/

/* Macros */
#define DISABLE_USB (USBCTL0 = 0x00)    // Disable USB module
#define ENDPOINTS   3

/* EP0 settings */
#define EP0_SIZE                0x40
#define GET_STATUS              0x00
#define CLEAR_FEATURE           0x01
#define SET_FEATURE             0x03

/********** Enpoint Value Options ***********/
/*                                          */
/* IN       Enabled for IN transactions     */
/* OUT      Enabled for OUT transactions    */
/* DISABLE  EndPoint Disabled               */
/*                                          */
/********************************************/


/* EP1 settings */
#define EP1_VALUE           _EP_IN
#define EP1_TYPE            INTERRUPT
#define EP1_SIZE            0x04
#define EP1_BUFF_OFFSET     0x40


/* EP2 settings */
#define EP2_VALUE           DISABLE
#define EP2_SIZE            1
#define EP2_BUFF_OFFSET     0x08

/* EP3 settings */
#define EP3_VALUE           DISABLE
#define EP3_SIZE            1
#define EP3_BUFF_OFFSET     0x08

/* EP4 settings */
#define EP4_VALUE           DISABLE
#define EP4_SIZE            1
#define EP4_BUFF_OFFSET     0x08

/* EP5 settings */
#define EP5_VALUE           DISABLE
#define EP5_SIZE            1
#define EP5_BUFF_OFFSET     0x08

/* EP6 settings */
#define EP6_VALUE           DISABLE
#define EP6_SIZE            1
#define EP6_BUFF_OFFSET     0x08

/* Macros */
#define EP3_CTR   tBDTtable[4].Stat._byte= kSIE 


/* MACROS */
#define usbSIE_CONTROL(EP)   (tBDTtable[EP<<2].Stat._byte= kSIE)
#define usbMCU_CONTROL(EP)   (tBDTtable[EP<<2].Stat._byte= kMCU)
#define usbEP_Reset(EP)      (tBDTtable[EP<<2].Cnt=0x0040)



#define _EP_IN      USB_ENDPT_EPTXEN_MASK
#define _EP_OUT     USB_ENDPT_EPRXEN_MASK




//USB_ENDPT_EPTXEN_MASK

//USB_ENDPT_EPRXEN_MASK

#define DISABLE 0

#define INTERRUPT   0x03
#define BULK        0x05


// BDT status value
#define kMCU      0x00
#define kSIE      0x80

#define kUDATA0   0x88
#define kUDATA1   0xC0

                            

#define mSETUP_TOKEN    0x0D
#define mOUT_TOKEN      0x01
#define mIN_TOKEN       0x09




    // USB commands
#define mGET_STATUS           0
#define mCLR_FEATURE          1
#define mSET_FEATURE          3
#define mSET_ADDRESS          5
#define mGET_DESC             6
#define mSET_DESC             7
#define mGET_CONFIG           8
#define mSET_CONFIG           9
#define mGET_INTF             10
#define mSET_INTF             11
#define mSYNC_FRAME           12
#define	mGET_MAXLUN	          0xFE		// Mass Storage command

#define mDEVICE					    1
#define mCONFIGURATION		        2
#define mSTRING					    3
#define mINTERFACE				    4
#define mENDPOINT				    5
#define	mDEVICE_QUALIFIER		    6
#define mOTHER_SPEED_CONFIGURATION	7
#define mINTERFACE_POWER			8





#define mREPORT                     0x22



/* Request Types */
#define STANDARD_REQ    0x00
#define SPECIFIC_REQ    0x20
#define VENDORSPEC_REQ  0x40
#define DEVICE_REQ      0x00
#define INTERFACE_REQ   0x01
#define ENDPOINT_REQ    0x02



enum
{
    uSETUP,
    uDATA
};

enum
{
    EP0,
    EP1,
    EP2,
    EP3,
    EP4,
    EP5,
    DUMMY,
    LOADER
    
};


enum
{
    uPOWER,
    uENUMERATED,
    uENABLED,
    uADDRESS,
    uREADY    
};
enum
{
    fIN,
    fOUT
};

enum
{
    bEP0OUT_ODD,
    bEP0OUT_EVEN,
    bEP0IN_ODD,
    bEP0IN_EVEN,
    bEP1OUT_ODD,
    bEP1OUT_EVEN,
    bEP1IN_ODD,
    bEP1IN_EVEN,
    bEP2OUT_ODD,
    bEP2OUT_EVEN,
    bEP2IN_ODD,
    bEP2IN_EVEN,
    bEP3OUT_ODD,
    bEP3OUT_EVEN,
    bEP3IN_ODD,
    bEP3IN_EVEN
};


/***** Data Types *****/
/*     */

    typedef struct{
        uint_8 _USB_PERID;
        uint_8 _USB_IDCOMP;
        uint_8 _USB_REV;
        uint_8 _USB_ADDINFO;
        uint_8 _USB_OTGISTAT;
        uint_8 _USB_OTGICR;
        uint_8 _USB_OTGSTAT;
        uint_8 _USB_OTGCTL;
        uint_8 _USB_ISTAT;
        uint_8 _USB_INTEN;
        uint_8 _USB_ERRSTAT;
        uint_8 _USB_ERREN;
        uint_8 _USB_STAT;
        uint_8 _USB_CTL;
        uint_8 _USB_ADDR;
        uint_8 _USB_BDTPAGE1;
        uint_8 _USB_FRMNUML;
        uint_8 _USB_FRMNUMH;
        uint_8 _USB_TOKEN;
        uint_8 _USB_SOFTHLD;
        uint_8 _USB_BDTPAGE2;
        uint_8 _USB_BDTPAGE3;
        uint_8 _USB_ENDPT0;
        uint_8 _USB_ENDPT1;
        uint_8 _USB_ENDPT2;
        uint_8 _USB_ENDPT3;
        uint_8 _USB_ENDPT4;
        uint_8 _USB_ENDPT5;
        uint_8 _USB_ENDPT6;
        uint_8 _USB_ENDPT7;
        uint_8 _USB_ENDPT8;
        uint_8 _USB_ENDPT9;
        uint_8 _USB_ENDPT10;
        uint_8 _USB_ENDPT11;
        uint_8 _USB_ENDPT12;
        uint_8 _USB_ENDPT13;
        uint_8 _USB_ENDPT14;
        uint_8 _USB_ENDPT15;
        uint_8 _USB_USBCTRL;
        uint_8 _USB_OBSERVE;
        uint_8 _USB_CONTROL;
        uint_8 _USB_USBTRC0;
    }USB_Registers;

typedef union _tBDT_STAT
{
    uint_8 _byte;
    struct{
        uint_8 :1;
        uint_8 :1;
        uint_8 BSTALL:1;              //Buffer Stall Enable
        uint_8 DTS:1;                 //Data Toggle Synch Enable
        uint_8 NINC:1;                //Address Increment Disable
        uint_8 KEEP:1;                //BD Keep Enable
        uint_8 DATA:1;                //Data Toggle Synch Value
        uint_8 UOWN:1;                //USB Ownership
    }McuCtlBit;
       
    struct{
        uint_8    :2;
        uint_8 PID:4;                 //Packet Identifier
        uint_8    :2;
    }RecPid;
} tBDT_STAT;                            //Buffer Descriptor Status Register




/* Coldfire */

typedef struct _tBDT
{
    tBDT_STAT Stat;
    uint_8  dummy;
    uint_16 Cnt;
    uint_32 Addr;             
} tBDT;                             





#define SWAP16(val)                                                 \
    (uint_16)((((uint_16)(val) >> 0x8) & 0xFF) |                    \
    (((uint_16)(val) & 0xFF) << 0x8))

#define SWAP32(val)                                                 \
    (uint_32)((SWAP16((uint_32)(val) & (uint_32)0xFFFF) << 0x10) |  \
    (SWAP16((uint_32)((val) >> 0x10))))


typedef struct _tUSB_Setup 
{
       uint_8 bmRequestType;
       uint_8 bRequest;
       uint_8 bValue_l;
       uint_8 bValue_h;
       uint_8 bIndex_l;
       uint_8 bIndex_h;
       uint_8 bLength_l;
       uint_8 bLength_h;
}tUSB_Setup;

/* bmRequestType */
#define USB_REQUESTTYPE_DATADIR(val)      (val & USB_REQUESTTYPE_DATADIR_MASK)
#define USB_REQUESTTYPE_TYPE(val)         (val & USB_REQUESTTYPE_TYPE_MASK)
#define USB_REQUESTTYPE_RECIPIENT(val)    (val & USB_REQUESTTYPE_RECIPIENT_MASK)

#define USB_REQUESTTYPE_DATADIR_MASK      (0X80)
#define USB_REQUESTTYPE_DATADIR_HOST2DEV  (0x00)
#define USB_REQUESTTYPE_DATADIR_DEV2HOST  (0x80)

#define USB_REQUESTTYPE_TYPE_MASK         (0X60)
#define USB_REQUESTTYPE_TYPE_STANDARD     (0x00)
#define USB_REQUESTTYPE_TYPE_CLASS        (0x20)
#define USB_REQUESTTYPE_TYPE_VENDOR       (0x40)

#define USB_REQUESTTYPE_RECIPIENT_MASK      (0x1F)
#define USB_REQUESTTYPE_RECIPIENT_DEVICE    (0x00)
#define USB_REQUESTTYPE_RECIPIENT_INTERFACE (0x01)
#define USB_REQUESTTYPE_RECIPIENT_ENDPOINT  (0x02)

/* wIndex_l */

/*****************************************************************************
 * Global Functions Prototypes
 *****************************************************************************/
extern void USB_Init(void);
/*****************************************************************************
 * Local Functions Prototypes
 *****************************************************************************/
void USB_Testing_Read_Registers(void);
void USB_Testing_Write_Registers(void);
void USB_Polling(void);
void USB_EP0_IN_Handler(void);
void EP_IN_Transfer(uint_8,uint_8*,uint_8);
uint_8 EP_OUT_Transfer(uint_8,uint_8*);
void USB_Stall_Handler(void);
void USB_Stall_Handler(void);
void USB_EP0_OUT_Handler(void);
void USB_EP0_Stall(void);
void USB_Set_Interface(void);
uint_16 USB_EP_OUT_SizeCheck(uint_8 u8EP);
void USB_Reset_Handler(void);
void Custom_Fields_Assignment(void);
void USB_Endpoint_Setup_Handler(void);
void USB_ISR(void);


#endif /* __USB__*/