/************************************************************************
* (c) Copyright Freescale Semiconductor, Inc 2010, All Rights Reserved  *
*************************************************************************

*************************************************************************
*                                                                       *
*        Standard Software Flash Driver For FTFx                        *
*                                                                       *
* FILE NAME     :  FlashCommandSequence.c                               *
* DATE          :  September 27,2010                                    *
*                                                                       *
* AUTHOR        :  FPT Team                                             *
* E-mail        :  b39392@freescale.com                                   *
*                                                                       *
*************************************************************************/

/************************** CHANGES *************************************
0.0.1       06.09.2010      FPT Team      Initial Version
0.1.0       06.11.2010      FPT Team      Finalize to 0.1.0 
0.1.3       09.16.2010      FPT Team      Finalize to 0.1.3 
0.2.0       09.27.2010      FPT Team      Finalize to 0.2.0
*************************************************************************/
/* include the header files */
#include "SSD_Types.h"
#include "SSD_FTFx.h"
#include "SSD_FTFx_Internal.h"

#if (defined(IAR))
	__ramfunc void FlashLaunchCommand (PFLASH_SSD_CONFIG PSSDConfig);

#elif (defined(__GNUC__))
void  __relocate_code__ FlashLaunchCommand (PFLASH_SSD_CONFIG PSSDConfig) ;
#endif

/************************************************************************
*
*  Function Name    : FlashCommandSequence
*  Description      : Perform command write sequence for flash operation
*  Arguments        : PFLASH_SSD_CONFIG, UINT8, UINT8*
*  Return Value     : UINT32
*
*************************************************************************/

UINT8 FTFx_SSD_FCCOB_OFFSET[] = {FTFx_SSD_FCCOB0_OFFSET,       \
                                  FTFx_SSD_FCCOB1_OFFSET,       \
                                  FTFx_SSD_FCCOB2_OFFSET,       \
                                  FTFx_SSD_FCCOB3_OFFSET,       \
                                  FTFx_SSD_FCCOB4_OFFSET,       \
                                  FTFx_SSD_FCCOB5_OFFSET,       \
                                  FTFx_SSD_FCCOB6_OFFSET,       \
                                  FTFx_SSD_FCCOB7_OFFSET,       \
                                  FTFx_SSD_FCCOB8_OFFSET,       \
                                  FTFx_SSD_FCCOB9_OFFSET,       \
                                  FTFx_SSD_FCCOBA_OFFSET,       \
                                  FTFx_SSD_FCCOBB_OFFSET       
                                };

UINT32 FlashCommandSequence (PFLASH_SSD_CONFIG PSSDConfig, \
                                         UINT8 index, \
                                         UINT8* pCommandArray)
{
    UINT8  counter;          /* for loop counter variable */
//    UINT8  registerValue;    /* store data read from flash register */
    UINT32 returnCode;       /* return code variable */

    /* set the default return as FTFx_OK */
    returnCode = FTFx_OK;

    /* check CCIF bit of the flash status register */
    
    while(FALSE == (REG_BIT_TEST(PSSDConfig->ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF)))
    {
        /* wait till CCIF bit is set */
    }

    /* clear RDCOLERR & ACCERR & FPVIOL flag in flash status register */
    REG_WRITE(PSSDConfig->ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, \
              (FTFx_SSD_FSTAT_RDCOLERR | FTFx_SSD_FSTAT_ACCERR | FTFx_SSD_FSTAT_FPVIOL));

    /* load FCCOB registers */
    for(counter=0; counter<=index; counter++)
    {
      REG_WRITE(PSSDConfig->ftfxRegBase + FTFx_SSD_FCCOB_OFFSET[counter], pCommandArray[counter]);

    }

    //call ram function to launch command
    FlashLaunchCommand(PSSDConfig);
    
     /* Check error bits */
    /*Get flash status register value */
  returnCode = REG_READ(PSSDConfig->ftfxRegBase + FTFx_SSD_FSTAT_OFFSET) & \
                (FTFx_SSD_FSTAT_ACCERR + FTFx_SSD_FSTAT_FPVIOL + FTFx_SSD_FSTAT_MGSTAT0 );

    return(returnCode);
}

#if (defined(IAR))
	__ramfunc void FlashLaunchCommand (PFLASH_SSD_CONFIG PSSDConfig)

#elif (defined(__GNUC__))
void __relocate_code__ FlashLaunchCommand (PFLASH_SSD_CONFIG PSSDConfig) 
#endif
{
 
	    /* clear CCIF bit */
	    REG_WRITE(PSSDConfig->ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF);
	    
	    /* check CCIF bit */
	    while (FALSE == (REG_BIT_TEST((PSSDConfig->ftfxRegBase + \
	           FTFx_SSD_FSTAT_OFFSET), FTFx_SSD_FSTAT_CCIF)))
    {
        /* wait till CCIF bit is set */

    }

} 

/* end of file */



